"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .applypatchservertool import ApplyPatchServerTool, ApplyPatchServerToolTypedDict
from .codeinterpreterservertool import (
    CodeInterpreterServerTool,
    CodeInterpreterServerToolTypedDict,
)
from .codexlocalshelltool import CodexLocalShellTool, CodexLocalShellToolTypedDict
from .computeruseservertool import ComputerUseServerTool, ComputerUseServerToolTypedDict
from .contextcompressionengine import ContextCompressionEngine
from .customtool import CustomTool, CustomToolTypedDict
from .datacollection import DataCollection
from .datetimeservertool import DatetimeServerTool, DatetimeServerToolTypedDict
from .filesearchservertool import FileSearchServerTool, FileSearchServerToolTypedDict
from .imagegenerationservertool import (
    ImageGenerationServerTool,
    ImageGenerationServerToolTypedDict,
)
from .inputs_union import InputsUnion, InputsUnionTypedDict
from .legacy_websearchservertool import (
    LegacyWebSearchServerTool,
    LegacyWebSearchServerToolTypedDict,
)
from .mcpservertool import McpServerTool, McpServerToolTypedDict
from .openairesponsestoolchoice_union import (
    OpenAIResponsesToolChoiceUnion,
    OpenAIResponsesToolChoiceUnionTypedDict,
)
from .openairesponsestruncation import OpenAIResponsesTruncation
from .outputmodalityenum import OutputModalityEnum
from .pdfparseroptions import PDFParserOptions, PDFParserOptionsTypedDict
from .preferredmaxlatency import PreferredMaxLatency, PreferredMaxLatencyTypedDict
from .preferredminthroughput import (
    PreferredMinThroughput,
    PreferredMinThroughputTypedDict,
)
from .preview_20250311_websearchservertool import (
    Preview20250311WebSearchServerTool,
    Preview20250311WebSearchServerToolTypedDict,
)
from .preview_websearchservertool import (
    PreviewWebSearchServerTool,
    PreviewWebSearchServerToolTypedDict,
)
from .providername import ProviderName
from .providersort import ProviderSort
from .providersortconfig import ProviderSortConfig, ProviderSortConfigTypedDict
from .quantization import Quantization
from .reasoningconfig import ReasoningConfig, ReasoningConfigTypedDict
from .responseincludesenum import ResponseIncludesEnum
from .shellservertool import ShellServerTool, ShellServerToolTypedDict
from .storedprompttemplate import StoredPromptTemplate, StoredPromptTemplateTypedDict
from .textextendedconfig import TextExtendedConfig, TextExtendedConfigTypedDict
from .websearchengine import WebSearchEngine
from .websearchservertool import WebSearchServerTool, WebSearchServerToolTypedDict
from .websearchservertool_openrouter import (
    WebSearchServerToolOpenRouter,
    WebSearchServerToolOpenRouterTypedDict,
)
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import get_discriminator, validate_const, validate_open_enum
import pydantic
from pydantic import ConfigDict, Discriminator, Tag, model_serializer
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Any, Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


ResponsesRequestType = Literal["function",]


class ResponsesRequestToolFunctionTypedDict(TypedDict):
    r"""Function tool definition"""

    type: ResponsesRequestType
    name: str
    parameters: Nullable[Dict[str, Nullable[Any]]]
    description: NotRequired[Nullable[str]]
    strict: NotRequired[Nullable[bool]]


class ResponsesRequestToolFunction(BaseModel):
    r"""Function tool definition"""

    type: ResponsesRequestType

    name: str

    parameters: Nullable[Dict[str, Nullable[Any]]]

    description: OptionalNullable[str] = UNSET

    strict: OptionalNullable[bool] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["description", "strict"]
        nullable_fields = ["description", "strict", "parameters"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


ResponsesRequestToolUnionTypedDict = TypeAliasType(
    "ResponsesRequestToolUnionTypedDict",
    Union[
        CodexLocalShellToolTypedDict,
        ApplyPatchServerToolTypedDict,
        ShellServerToolTypedDict,
        WebSearchServerToolOpenRouterTypedDict,
        DatetimeServerToolTypedDict,
        CodeInterpreterServerToolTypedDict,
        CustomToolTypedDict,
        ComputerUseServerToolTypedDict,
        ResponsesRequestToolFunctionTypedDict,
        FileSearchServerToolTypedDict,
        WebSearchServerToolTypedDict,
        LegacyWebSearchServerToolTypedDict,
        Preview20250311WebSearchServerToolTypedDict,
        PreviewWebSearchServerToolTypedDict,
        McpServerToolTypedDict,
        ImageGenerationServerToolTypedDict,
    ],
)


ResponsesRequestToolUnion = Annotated[
    Union[
        Annotated[ResponsesRequestToolFunction, Tag("function")],
        Annotated[PreviewWebSearchServerTool, Tag("web_search_preview")],
        Annotated[
            Preview20250311WebSearchServerTool, Tag("web_search_preview_2025_03_11")
        ],
        Annotated[LegacyWebSearchServerTool, Tag("web_search")],
        Annotated[WebSearchServerTool, Tag("web_search_2025_08_26")],
        Annotated[FileSearchServerTool, Tag("file_search")],
        Annotated[ComputerUseServerTool, Tag("computer_use_preview")],
        Annotated[CodeInterpreterServerTool, Tag("code_interpreter")],
        Annotated[McpServerTool, Tag("mcp")],
        Annotated[ImageGenerationServerTool, Tag("image_generation")],
        Annotated[CodexLocalShellTool, Tag("local_shell")],
        Annotated[ShellServerTool, Tag("shell")],
        Annotated[ApplyPatchServerTool, Tag("apply_patch")],
        Annotated[CustomTool, Tag("custom")],
        Annotated[DatetimeServerTool, Tag("openrouter:datetime")],
        Annotated[WebSearchServerToolOpenRouter, Tag("openrouter:web_search")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


ResponsesRequestImageConfigTypedDict = TypeAliasType(
    "ResponsesRequestImageConfigTypedDict", Union[str, float]
)


ResponsesRequestImageConfig = TypeAliasType(
    "ResponsesRequestImageConfig", Union[str, float]
)


ResponsesRequestServiceTier = Union[
    Literal[
        "auto",
        "default",
        "flex",
        "priority",
        "scale",
    ],
    UnrecognizedStr,
]


ResponsesRequestOrderTypedDict = TypeAliasType(
    "ResponsesRequestOrderTypedDict", Union[ProviderName, str]
)


ResponsesRequestOrder = TypeAliasType(
    "ResponsesRequestOrder",
    Union[Annotated[ProviderName, PlainValidator(validate_open_enum(False))], str],
)


ResponsesRequestOnlyTypedDict = TypeAliasType(
    "ResponsesRequestOnlyTypedDict", Union[ProviderName, str]
)


ResponsesRequestOnly = TypeAliasType(
    "ResponsesRequestOnly",
    Union[Annotated[ProviderName, PlainValidator(validate_open_enum(False))], str],
)


ResponsesRequestIgnoreTypedDict = TypeAliasType(
    "ResponsesRequestIgnoreTypedDict", Union[ProviderName, str]
)


ResponsesRequestIgnore = TypeAliasType(
    "ResponsesRequestIgnore",
    Union[Annotated[ProviderName, PlainValidator(validate_open_enum(False))], str],
)


ResponsesRequestSortTypedDict = TypeAliasType(
    "ResponsesRequestSortTypedDict",
    Union[ProviderSortConfigTypedDict, ProviderSort, Any],
)
r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""


ResponsesRequestSort = TypeAliasType(
    "ResponsesRequestSort",
    Union[
        ProviderSortConfig,
        Annotated[ProviderSort, PlainValidator(validate_open_enum(False))],
        Any,
    ],
)
r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""


class ResponsesRequestMaxPriceTypedDict(TypedDict):
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    prompt: NotRequired[str]
    r"""Price per million prompt tokens"""
    completion: NotRequired[str]
    image: NotRequired[str]
    audio: NotRequired[str]
    request: NotRequired[str]


class ResponsesRequestMaxPrice(BaseModel):
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    prompt: Optional[str] = None
    r"""Price per million prompt tokens"""

    completion: Optional[str] = None

    image: Optional[str] = None

    audio: Optional[str] = None

    request: Optional[str] = None


class ResponsesRequestProviderTypedDict(TypedDict):
    r"""When multiple model providers are available, optionally indicate your routing preference."""

    allow_fallbacks: NotRequired[Nullable[bool]]
    r"""Whether to allow backup providers to serve requests
    - true: (default) when the primary provider (or your custom providers in \"order\") is unavailable, use the next best provider.
    - false: use only the primary/custom provider, and return the upstream error if it's unavailable.

    """
    require_parameters: NotRequired[Nullable[bool]]
    r"""Whether to filter providers to only those that support the parameters you've provided. If this setting is omitted or set to false, then providers will receive only the parameters they support, and ignore the rest."""
    data_collection: NotRequired[Nullable[DataCollection]]
    r"""Data collection setting. If no available model provider meets the requirement, your request will return an error.
    - allow: (default) allow providers which store user data non-transiently and may train on it

    - deny: use only providers which do not collect user data.
    """
    zdr: NotRequired[Nullable[bool]]
    r"""Whether to restrict routing to only ZDR (Zero Data Retention) endpoints. When true, only endpoints that do not retain prompts will be used."""
    enforce_distillable_text: NotRequired[Nullable[bool]]
    r"""Whether to restrict routing to only models that allow text distillation. When true, only models where the author has allowed distillation will be used."""
    order: NotRequired[Nullable[List[ResponsesRequestOrderTypedDict]]]
    r"""An ordered list of provider slugs. The router will attempt to use the first provider in the subset of this list that supports your requested model, and fall back to the next if it is unavailable. If no providers are available, the request will fail with an error message."""
    only: NotRequired[Nullable[List[ResponsesRequestOnlyTypedDict]]]
    r"""List of provider slugs to allow. If provided, this list is merged with your account-wide allowed provider settings for this request."""
    ignore: NotRequired[Nullable[List[ResponsesRequestIgnoreTypedDict]]]
    r"""List of provider slugs to ignore. If provided, this list is merged with your account-wide ignored provider settings for this request."""
    quantizations: NotRequired[Nullable[List[Quantization]]]
    r"""A list of quantization levels to filter the provider by."""
    sort: NotRequired[Nullable[ResponsesRequestSortTypedDict]]
    r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""
    max_price: NotRequired[ResponsesRequestMaxPriceTypedDict]
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""
    preferred_min_throughput: NotRequired[Nullable[PreferredMinThroughputTypedDict]]
    r"""Preferred minimum throughput (in tokens per second). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints below the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""
    preferred_max_latency: NotRequired[Nullable[PreferredMaxLatencyTypedDict]]
    r"""Preferred maximum latency (in seconds). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints above the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""


class ResponsesRequestProvider(BaseModel):
    r"""When multiple model providers are available, optionally indicate your routing preference."""

    allow_fallbacks: OptionalNullable[bool] = UNSET
    r"""Whether to allow backup providers to serve requests
    - true: (default) when the primary provider (or your custom providers in \"order\") is unavailable, use the next best provider.
    - false: use only the primary/custom provider, and return the upstream error if it's unavailable.

    """

    require_parameters: OptionalNullable[bool] = UNSET
    r"""Whether to filter providers to only those that support the parameters you've provided. If this setting is omitted or set to false, then providers will receive only the parameters they support, and ignore the rest."""

    data_collection: Annotated[
        OptionalNullable[DataCollection], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""Data collection setting. If no available model provider meets the requirement, your request will return an error.
    - allow: (default) allow providers which store user data non-transiently and may train on it

    - deny: use only providers which do not collect user data.
    """

    zdr: OptionalNullable[bool] = UNSET
    r"""Whether to restrict routing to only ZDR (Zero Data Retention) endpoints. When true, only endpoints that do not retain prompts will be used."""

    enforce_distillable_text: OptionalNullable[bool] = UNSET
    r"""Whether to restrict routing to only models that allow text distillation. When true, only models where the author has allowed distillation will be used."""

    order: OptionalNullable[List[ResponsesRequestOrder]] = UNSET
    r"""An ordered list of provider slugs. The router will attempt to use the first provider in the subset of this list that supports your requested model, and fall back to the next if it is unavailable. If no providers are available, the request will fail with an error message."""

    only: OptionalNullable[List[ResponsesRequestOnly]] = UNSET
    r"""List of provider slugs to allow. If provided, this list is merged with your account-wide allowed provider settings for this request."""

    ignore: OptionalNullable[List[ResponsesRequestIgnore]] = UNSET
    r"""List of provider slugs to ignore. If provided, this list is merged with your account-wide ignored provider settings for this request."""

    quantizations: OptionalNullable[
        List[Annotated[Quantization, PlainValidator(validate_open_enum(False))]]
    ] = UNSET
    r"""A list of quantization levels to filter the provider by."""

    sort: OptionalNullable[ResponsesRequestSort] = UNSET
    r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""

    max_price: Optional[ResponsesRequestMaxPrice] = None
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    preferred_min_throughput: OptionalNullable[PreferredMinThroughput] = UNSET
    r"""Preferred minimum throughput (in tokens per second). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints below the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""

    preferred_max_latency: OptionalNullable[PreferredMaxLatency] = UNSET
    r"""Preferred maximum latency (in seconds). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints above the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "allow_fallbacks",
            "require_parameters",
            "data_collection",
            "zdr",
            "enforce_distillable_text",
            "order",
            "only",
            "ignore",
            "quantizations",
            "sort",
            "max_price",
            "preferred_min_throughput",
            "preferred_max_latency",
        ]
        nullable_fields = [
            "allow_fallbacks",
            "require_parameters",
            "data_collection",
            "zdr",
            "enforce_distillable_text",
            "order",
            "only",
            "ignore",
            "quantizations",
            "sort",
            "preferred_min_throughput",
            "preferred_max_latency",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


ResponsesRequestIDContextCompression = Literal["context-compression",]


class ResponsesRequestPluginContextCompressionTypedDict(TypedDict):
    id: ResponsesRequestIDContextCompression
    enabled: NotRequired[bool]
    r"""Set to false to disable the context-compression plugin for this request. Defaults to true."""
    engine: NotRequired[ContextCompressionEngine]
    r"""The compression engine to use. Defaults to \"middle-out\"."""


class ResponsesRequestPluginContextCompression(BaseModel):
    id: ResponsesRequestIDContextCompression

    enabled: Optional[bool] = None
    r"""Set to false to disable the context-compression plugin for this request. Defaults to true."""

    engine: Optional[ContextCompressionEngine] = None
    r"""The compression engine to use. Defaults to \"middle-out\"."""


ResponsesRequestIDResponseHealing = Literal["response-healing",]


class ResponsesRequestPluginResponseHealingTypedDict(TypedDict):
    id: ResponsesRequestIDResponseHealing
    enabled: NotRequired[bool]
    r"""Set to false to disable the response-healing plugin for this request. Defaults to true."""


class ResponsesRequestPluginResponseHealing(BaseModel):
    id: ResponsesRequestIDResponseHealing

    enabled: Optional[bool] = None
    r"""Set to false to disable the response-healing plugin for this request. Defaults to true."""


ResponsesRequestIDFileParser = Literal["file-parser",]


class ResponsesRequestPluginFileParserTypedDict(TypedDict):
    id: ResponsesRequestIDFileParser
    enabled: NotRequired[bool]
    r"""Set to false to disable the file-parser plugin for this request. Defaults to true."""
    pdf: NotRequired[PDFParserOptionsTypedDict]
    r"""Options for PDF parsing."""


class ResponsesRequestPluginFileParser(BaseModel):
    id: ResponsesRequestIDFileParser

    enabled: Optional[bool] = None
    r"""Set to false to disable the file-parser plugin for this request. Defaults to true."""

    pdf: Optional[PDFParserOptions] = None
    r"""Options for PDF parsing."""


ResponsesRequestIDWeb = Literal["web",]


class ResponsesRequestPluginWebTypedDict(TypedDict):
    id: ResponsesRequestIDWeb
    enabled: NotRequired[bool]
    r"""Set to false to disable the web-search plugin for this request. Defaults to true."""
    max_results: NotRequired[float]
    search_prompt: NotRequired[str]
    engine: NotRequired[WebSearchEngine]
    r"""The search engine to use for web search."""
    include_domains: NotRequired[List[str]]
    r"""A list of domains to restrict web search results to. Supports wildcards (e.g. \"*.substack.com\") and path filtering (e.g. \"openai.com/blog\")."""
    exclude_domains: NotRequired[List[str]]
    r"""A list of domains to exclude from web search results. Supports wildcards (e.g. \"*.substack.com\") and path filtering (e.g. \"openai.com/blog\")."""


class ResponsesRequestPluginWeb(BaseModel):
    id: ResponsesRequestIDWeb

    enabled: Optional[bool] = None
    r"""Set to false to disable the web-search plugin for this request. Defaults to true."""

    max_results: Optional[float] = None

    search_prompt: Optional[str] = None

    engine: Annotated[
        Optional[WebSearchEngine], PlainValidator(validate_open_enum(False))
    ] = None
    r"""The search engine to use for web search."""

    include_domains: Optional[List[str]] = None
    r"""A list of domains to restrict web search results to. Supports wildcards (e.g. \"*.substack.com\") and path filtering (e.g. \"openai.com/blog\")."""

    exclude_domains: Optional[List[str]] = None
    r"""A list of domains to exclude from web search results. Supports wildcards (e.g. \"*.substack.com\") and path filtering (e.g. \"openai.com/blog\")."""


ResponsesRequestIDModeration = Literal["moderation",]


class ResponsesRequestPluginModerationTypedDict(TypedDict):
    id: ResponsesRequestIDModeration


class ResponsesRequestPluginModeration(BaseModel):
    id: ResponsesRequestIDModeration


ResponsesRequestIDAutoRouter = Literal["auto-router",]


class ResponsesRequestPluginAutoRouterTypedDict(TypedDict):
    id: ResponsesRequestIDAutoRouter
    enabled: NotRequired[bool]
    r"""Set to false to disable the auto-router plugin for this request. Defaults to true."""
    allowed_models: NotRequired[List[str]]
    r"""List of model patterns to filter which models the auto-router can route between. Supports wildcards (e.g., \"anthropic/*\" matches all Anthropic models). When not specified, uses the default supported models list."""


class ResponsesRequestPluginAutoRouter(BaseModel):
    id: ResponsesRequestIDAutoRouter

    enabled: Optional[bool] = None
    r"""Set to false to disable the auto-router plugin for this request. Defaults to true."""

    allowed_models: Optional[List[str]] = None
    r"""List of model patterns to filter which models the auto-router can route between. Supports wildcards (e.g., \"anthropic/*\" matches all Anthropic models). When not specified, uses the default supported models list."""


ResponsesRequestPluginUnionTypedDict = TypeAliasType(
    "ResponsesRequestPluginUnionTypedDict",
    Union[
        ResponsesRequestPluginModerationTypedDict,
        ResponsesRequestPluginResponseHealingTypedDict,
        ResponsesRequestPluginAutoRouterTypedDict,
        ResponsesRequestPluginFileParserTypedDict,
        ResponsesRequestPluginContextCompressionTypedDict,
        ResponsesRequestPluginWebTypedDict,
    ],
)


ResponsesRequestPluginUnion = Annotated[
    Union[
        Annotated[ResponsesRequestPluginAutoRouter, Tag("auto-router")],
        Annotated[ResponsesRequestPluginModeration, Tag("moderation")],
        Annotated[ResponsesRequestPluginWeb, Tag("web")],
        Annotated[ResponsesRequestPluginFileParser, Tag("file-parser")],
        Annotated[ResponsesRequestPluginResponseHealing, Tag("response-healing")],
        Annotated[ResponsesRequestPluginContextCompression, Tag("context-compression")],
    ],
    Discriminator(lambda m: get_discriminator(m, "id", "id")),
]


class ResponsesRequestTraceTypedDict(TypedDict):
    r"""Metadata for observability and tracing. Known keys (trace_id, trace_name, span_name, generation_name, parent_span_id) have special handling. Additional keys are passed through as custom metadata to configured broadcast destinations."""

    trace_id: NotRequired[str]
    trace_name: NotRequired[str]
    span_name: NotRequired[str]
    generation_name: NotRequired[str]
    parent_span_id: NotRequired[str]


class ResponsesRequestTrace(BaseModel):
    r"""Metadata for observability and tracing. Known keys (trace_id, trace_name, span_name, generation_name, parent_span_id) have special handling. Additional keys are passed through as custom metadata to configured broadcast destinations."""

    model_config = ConfigDict(
        populate_by_name=True, arbitrary_types_allowed=True, extra="allow"
    )
    __pydantic_extra__: Dict[str, Nullable[Any]] = pydantic.Field(init=False)

    trace_id: Optional[str] = None

    trace_name: Optional[str] = None

    span_name: Optional[str] = None

    generation_name: Optional[str] = None

    parent_span_id: Optional[str] = None

    @property
    def additional_properties(self):
        return self.__pydantic_extra__

    @additional_properties.setter
    def additional_properties(self, value):
        self.__pydantic_extra__ = value  # pyright: ignore[reportIncompatibleVariableOverride]


class ResponsesRequestTypedDict(TypedDict):
    r"""Request schema for Responses endpoint"""

    input: NotRequired[InputsUnionTypedDict]
    r"""Input for a response request - can be a string or array of items"""
    instructions: NotRequired[Nullable[str]]
    metadata: NotRequired[Nullable[Dict[str, str]]]
    r"""Metadata key-value pairs for the request. Keys must be ≤64 characters and cannot contain brackets. Values must be ≤512 characters. Maximum 16 pairs allowed."""
    tools: NotRequired[List[ResponsesRequestToolUnionTypedDict]]
    tool_choice: NotRequired[OpenAIResponsesToolChoiceUnionTypedDict]
    parallel_tool_calls: NotRequired[Nullable[bool]]
    model: NotRequired[str]
    models: NotRequired[List[str]]
    text: NotRequired[TextExtendedConfigTypedDict]
    r"""Text output configuration including format and verbosity"""
    reasoning: NotRequired[Nullable[ReasoningConfigTypedDict]]
    r"""Configuration for reasoning mode in the response"""
    max_output_tokens: NotRequired[Nullable[float]]
    temperature: NotRequired[Nullable[float]]
    top_p: NotRequired[Nullable[float]]
    top_logprobs: NotRequired[Nullable[int]]
    max_tool_calls: NotRequired[Nullable[int]]
    presence_penalty: NotRequired[Nullable[float]]
    frequency_penalty: NotRequired[Nullable[float]]
    top_k: NotRequired[float]
    image_config: NotRequired[Dict[str, ResponsesRequestImageConfigTypedDict]]
    r"""Provider-specific image configuration options. Keys and values vary by model/provider. See https://openrouter.ai/docs/features/multimodal/image-generation for more details."""
    modalities: NotRequired[List[OutputModalityEnum]]
    r"""Output modalities for the response. Supported values are \"text\" and \"image\"."""
    prompt_cache_key: NotRequired[Nullable[str]]
    previous_response_id: NotRequired[Nullable[str]]
    prompt: NotRequired[Nullable[StoredPromptTemplateTypedDict]]
    include: NotRequired[Nullable[List[ResponseIncludesEnum]]]
    background: NotRequired[Nullable[bool]]
    safety_identifier: NotRequired[Nullable[str]]
    store: Literal[False]
    service_tier: NotRequired[Nullable[ResponsesRequestServiceTier]]
    truncation: NotRequired[Nullable[OpenAIResponsesTruncation]]
    stream: NotRequired[bool]
    provider: NotRequired[Nullable[ResponsesRequestProviderTypedDict]]
    r"""When multiple model providers are available, optionally indicate your routing preference."""
    plugins: NotRequired[List[ResponsesRequestPluginUnionTypedDict]]
    r"""Plugins you want to enable for this request, including their settings."""
    user: NotRequired[str]
    r"""A unique identifier representing your end-user, which helps distinguish between different users of your app. This allows your app to identify specific users in case of abuse reports, preventing your entire app from being affected by the actions of individual users. Maximum of 256 characters."""
    session_id: NotRequired[str]
    r"""A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 256 characters."""
    trace: NotRequired[ResponsesRequestTraceTypedDict]
    r"""Metadata for observability and tracing. Known keys (trace_id, trace_name, span_name, generation_name, parent_span_id) have special handling. Additional keys are passed through as custom metadata to configured broadcast destinations."""


class ResponsesRequest(BaseModel):
    r"""Request schema for Responses endpoint"""

    input: Optional[InputsUnion] = None
    r"""Input for a response request - can be a string or array of items"""

    instructions: OptionalNullable[str] = UNSET

    metadata: OptionalNullable[Dict[str, str]] = UNSET
    r"""Metadata key-value pairs for the request. Keys must be ≤64 characters and cannot contain brackets. Values must be ≤512 characters. Maximum 16 pairs allowed."""

    tools: Optional[List[ResponsesRequestToolUnion]] = None

    tool_choice: Optional[OpenAIResponsesToolChoiceUnion] = None

    parallel_tool_calls: OptionalNullable[bool] = UNSET

    model: Optional[str] = None

    models: Optional[List[str]] = None

    text: Optional[TextExtendedConfig] = None
    r"""Text output configuration including format and verbosity"""

    reasoning: OptionalNullable[ReasoningConfig] = UNSET
    r"""Configuration for reasoning mode in the response"""

    max_output_tokens: OptionalNullable[float] = UNSET

    temperature: OptionalNullable[float] = UNSET

    top_p: OptionalNullable[float] = UNSET

    top_logprobs: OptionalNullable[int] = UNSET

    max_tool_calls: OptionalNullable[int] = UNSET

    presence_penalty: OptionalNullable[float] = UNSET

    frequency_penalty: OptionalNullable[float] = UNSET

    top_k: Optional[float] = None

    image_config: Optional[Dict[str, ResponsesRequestImageConfig]] = None
    r"""Provider-specific image configuration options. Keys and values vary by model/provider. See https://openrouter.ai/docs/features/multimodal/image-generation for more details."""

    modalities: Optional[
        List[Annotated[OutputModalityEnum, PlainValidator(validate_open_enum(False))]]
    ] = None
    r"""Output modalities for the response. Supported values are \"text\" and \"image\"."""

    prompt_cache_key: OptionalNullable[str] = UNSET

    previous_response_id: OptionalNullable[str] = UNSET

    prompt: OptionalNullable[StoredPromptTemplate] = UNSET

    include: OptionalNullable[
        List[Annotated[ResponseIncludesEnum, PlainValidator(validate_open_enum(False))]]
    ] = UNSET

    background: OptionalNullable[bool] = UNSET

    safety_identifier: OptionalNullable[str] = UNSET

    STORE: Annotated[
        Annotated[Optional[Literal[False]], AfterValidator(validate_const(False))],
        pydantic.Field(alias="store"),
    ] = False

    service_tier: Annotated[
        OptionalNullable[ResponsesRequestServiceTier],
        PlainValidator(validate_open_enum(False)),
    ] = "auto"

    truncation: Annotated[
        OptionalNullable[OpenAIResponsesTruncation],
        PlainValidator(validate_open_enum(False)),
    ] = UNSET

    stream: Optional[bool] = False

    provider: OptionalNullable[ResponsesRequestProvider] = UNSET
    r"""When multiple model providers are available, optionally indicate your routing preference."""

    plugins: Optional[List[ResponsesRequestPluginUnion]] = None
    r"""Plugins you want to enable for this request, including their settings."""

    user: Optional[str] = None
    r"""A unique identifier representing your end-user, which helps distinguish between different users of your app. This allows your app to identify specific users in case of abuse reports, preventing your entire app from being affected by the actions of individual users. Maximum of 256 characters."""

    session_id: Optional[str] = None
    r"""A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 256 characters."""

    trace: Optional[ResponsesRequestTrace] = None
    r"""Metadata for observability and tracing. Known keys (trace_id, trace_name, span_name, generation_name, parent_span_id) have special handling. Additional keys are passed through as custom metadata to configured broadcast destinations."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "input",
            "instructions",
            "metadata",
            "tools",
            "tool_choice",
            "parallel_tool_calls",
            "model",
            "models",
            "text",
            "reasoning",
            "max_output_tokens",
            "temperature",
            "top_p",
            "top_logprobs",
            "max_tool_calls",
            "presence_penalty",
            "frequency_penalty",
            "top_k",
            "image_config",
            "modalities",
            "prompt_cache_key",
            "previous_response_id",
            "prompt",
            "include",
            "background",
            "safety_identifier",
            "store",
            "service_tier",
            "truncation",
            "stream",
            "provider",
            "plugins",
            "user",
            "session_id",
            "trace",
        ]
        nullable_fields = [
            "instructions",
            "metadata",
            "parallel_tool_calls",
            "reasoning",
            "max_output_tokens",
            "temperature",
            "top_p",
            "top_logprobs",
            "max_tool_calls",
            "presence_penalty",
            "frequency_penalty",
            "prompt_cache_key",
            "previous_response_id",
            "prompt",
            "include",
            "background",
            "safety_identifier",
            "service_tier",
            "truncation",
            "provider",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


ChatContentFileType = Literal["file",]


class FileTypedDict(TypedDict):
    file_data: NotRequired[str]
    r"""File content as base64 data URL or URL"""
    file_id: NotRequired[str]
    r"""File ID for previously uploaded files"""
    filename: NotRequired[str]
    r"""Original filename"""


class File(BaseModel):
    file_data: Optional[str] = None
    r"""File content as base64 data URL or URL"""

    file_id: Optional[str] = None
    r"""File ID for previously uploaded files"""

    filename: Optional[str] = None
    r"""Original filename"""


class ChatContentFileTypedDict(TypedDict):
    r"""File content part for document processing"""

    type: ChatContentFileType
    file: FileTypedDict


class ChatContentFile(BaseModel):
    r"""File content part for document processing"""

    type: ChatContentFileType

    file: File



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ChatCompletionAudioOutputTypedDict(TypedDict):
    r"""Audio output data or reference"""

    id: NotRequired[str]
    r"""Audio output identifier"""
    expires_at: NotRequired[float]
    r"""Audio expiration timestamp"""
    data: NotRequired[str]
    r"""Base64 encoded audio data"""
    transcript: NotRequired[str]
    r"""Audio transcript"""


class ChatCompletionAudioOutput(BaseModel):
    r"""Audio output data or reference"""

    id: Optional[str] = None
    r"""Audio output identifier"""

    expires_at: Optional[float] = None
    r"""Audio expiration timestamp"""

    data: Optional[str] = None
    r"""Base64 encoded audio data"""

    transcript: Optional[str] = None
    r"""Audio transcript"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing_extensions import TypedDict


class AssistantMessageImagesImageURLTypedDict(TypedDict):
    url: str
    r"""URL or base64-encoded data of the generated image"""


class AssistantMessageImagesImageURL(BaseModel):
    url: str
    r"""URL or base64-encoded data of the generated image"""


class AssistantMessageImagesTypedDict(TypedDict):
    image_url: AssistantMessageImagesImageURLTypedDict


class AssistantMessageImages(BaseModel):
    image_url: AssistantMessageImagesImageURL



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CompletionTokensDetailsTypedDict(TypedDict):
    r"""Detailed completion token usage"""

    reasoning_tokens: NotRequired[Nullable[float]]
    r"""Tokens used for reasoning"""
    audio_tokens: NotRequired[Nullable[float]]
    r"""Tokens used for audio output"""
    accepted_prediction_tokens: NotRequired[Nullable[float]]
    r"""Accepted prediction tokens"""
    rejected_prediction_tokens: NotRequired[Nullable[float]]
    r"""Rejected prediction tokens"""


class CompletionTokensDetails(BaseModel):
    r"""Detailed completion token usage"""

    reasoning_tokens: OptionalNullable[float] = UNSET
    r"""Tokens used for reasoning"""

    audio_tokens: OptionalNullable[float] = UNSET
    r"""Tokens used for audio output"""

    accepted_prediction_tokens: OptionalNullable[float] = UNSET
    r"""Accepted prediction tokens"""

    rejected_prediction_tokens: OptionalNullable[float] = UNSET
    r"""Rejected prediction tokens"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "reasoning_tokens",
            "audio_tokens",
            "accepted_prediction_tokens",
            "rejected_prediction_tokens",
        ]
        nullable_fields = [
            "reasoning_tokens",
            "audio_tokens",
            "accepted_prediction_tokens",
            "rejected_prediction_tokens",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class PromptTokensDetailsTypedDict(TypedDict):
    r"""Detailed prompt token usage"""

    cached_tokens: NotRequired[float]
    r"""Cached prompt tokens"""
    cache_write_tokens: NotRequired[float]
    r"""Tokens written to cache. Only returned for models with explicit caching and cache write pricing."""
    audio_tokens: NotRequired[float]
    r"""Audio input tokens"""
    video_tokens: NotRequired[float]
    r"""Video input tokens"""


class PromptTokensDetails(BaseModel):
    r"""Detailed prompt token usage"""

    cached_tokens: Optional[float] = None
    r"""Cached prompt tokens"""

    cache_write_tokens: Optional[float] = None
    r"""Tokens written to cache. Only returned for models with explicit caching and cache write pricing."""

    audio_tokens: Optional[float] = None
    r"""Audio input tokens"""

    video_tokens: Optional[float] = None
    r"""Video input tokens"""


class ChatUsageTypedDict(TypedDict):
    r"""Token usage statistics"""

    completion_tokens: float
    r"""Number of tokens in the completion"""
    prompt_tokens: float
    r"""Number of tokens in the prompt"""
    total_tokens: float
    r"""Total number of tokens"""
    completion_tokens_details: NotRequired[Nullable[CompletionTokensDetailsTypedDict]]
    r"""Detailed completion token usage"""
    prompt_tokens_details: NotRequired[Nullable[PromptTokensDetailsTypedDict]]
    r"""Detailed prompt token usage"""


class ChatUsage(BaseModel):
    r"""Token usage statistics"""

    completion_tokens: float
    r"""Number of tokens in the completion"""

    prompt_tokens: float
    r"""Number of tokens in the prompt"""

    total_tokens: float
    r"""Total number of tokens"""

    completion_tokens_details: OptionalNullable[CompletionTokensDetails] = UNSET
    r"""Detailed completion token usage"""

    prompt_tokens_details: OptionalNullable[PromptTokensDetails] = UNSET
    r"""Detailed prompt token usage"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["completion_tokens_details", "prompt_tokens_details"]
        nullable_fields = ["completion_tokens_details", "prompt_tokens_details"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


OpenResponsesApplyPatchToolType = Literal["apply_patch",]


class OpenResponsesApplyPatchToolTypedDict(TypedDict):
    r"""Apply patch tool configuration"""

    type: OpenResponsesApplyPatchToolType


class OpenResponsesApplyPatchTool(BaseModel):
    r"""Apply patch tool configuration"""

    type: OpenResponsesApplyPatchToolType



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ResponseFormatTextPythonType = Literal["python",]


class ResponseFormatTextPythonTypedDict(TypedDict):
    r"""Python code response format"""

    type: ResponseFormatTextPythonType


class ResponseFormatTextPython(BaseModel):
    r"""Python code response format"""

    type: ResponseFormatTextPythonType



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


InputTextType = Literal["input_text",]


class InputTextTypedDict(TypedDict):
    r"""Text input content item"""

    type: InputTextType
    text: str


class InputText(BaseModel):
    r"""Text input content item"""

    type: InputTextType

    text: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .openairesponsesrefusalcontent import (
    OpenAIResponsesRefusalContent,
    OpenAIResponsesRefusalContentTypedDict,
)
from .responseoutputtext import ResponseOutputText, ResponseOutputTextTypedDict
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from openrouter.utils import get_discriminator
from pydantic import Discriminator, Tag, model_serializer
from typing import Any, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


OutputMessageRole = Literal["assistant",]


OutputMessageType = Literal["message",]


OutputMessageStatusInProgress = Literal["in_progress",]


OutputMessageStatusIncomplete = Literal["incomplete",]


OutputMessageStatusCompleted = Literal["completed",]


OutputMessageStatusUnionTypedDict = TypeAliasType(
    "OutputMessageStatusUnionTypedDict",
    Union[
        OutputMessageStatusCompleted,
        OutputMessageStatusIncomplete,
        OutputMessageStatusInProgress,
    ],
)


OutputMessageStatusUnion = TypeAliasType(
    "OutputMessageStatusUnion",
    Union[
        OutputMessageStatusCompleted,
        OutputMessageStatusIncomplete,
        OutputMessageStatusInProgress,
    ],
)


OutputMessageContentTypedDict = TypeAliasType(
    "OutputMessageContentTypedDict",
    Union[OpenAIResponsesRefusalContentTypedDict, ResponseOutputTextTypedDict],
)


OutputMessageContent = Annotated[
    Union[
        Annotated[ResponseOutputText, Tag("output_text")],
        Annotated[OpenAIResponsesRefusalContent, Tag("refusal")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


OutputMessagePhaseFinalAnswer = Literal["final_answer",]


OutputMessagePhaseCommentary = Literal["commentary",]


OutputMessagePhaseUnionTypedDict = TypeAliasType(
    "OutputMessagePhaseUnionTypedDict",
    Union[OutputMessagePhaseCommentary, OutputMessagePhaseFinalAnswer, Any],
)
r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""


OutputMessagePhaseUnion = TypeAliasType(
    "OutputMessagePhaseUnion",
    Union[OutputMessagePhaseCommentary, OutputMessagePhaseFinalAnswer, Any],
)
r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""


class OutputMessageTypedDict(TypedDict):
    id: str
    role: OutputMessageRole
    type: OutputMessageType
    content: List[OutputMessageContentTypedDict]
    status: NotRequired[OutputMessageStatusUnionTypedDict]
    phase: NotRequired[Nullable[OutputMessagePhaseUnionTypedDict]]
    r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""


class OutputMessage(BaseModel):
    id: str

    role: OutputMessageRole

    type: OutputMessageType

    content: List[OutputMessageContent]

    status: Optional[OutputMessageStatusUnion] = None

    phase: OptionalNullable[OutputMessagePhaseUnion] = UNSET
    r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["status", "phase"]
        nullable_fields = ["phase"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, UnrecognizedStr
from openrouter.utils import validate_open_enum
from pydantic.functional_validators import PlainValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


ChatContentImageType = Literal["image_url",]


ChatContentImageDetail = Union[
    Literal[
        "auto",
        "low",
        "high",
    ],
    UnrecognizedStr,
]
r"""Image detail level for vision models"""


class ChatContentImageImageURLTypedDict(TypedDict):
    url: str
    r"""URL of the image (data: URLs supported)"""
    detail: NotRequired[ChatContentImageDetail]
    r"""Image detail level for vision models"""


class ChatContentImageImageURL(BaseModel):
    url: str
    r"""URL of the image (data: URLs supported)"""

    detail: Annotated[
        Optional[ChatContentImageDetail], PlainValidator(validate_open_enum(False))
    ] = None
    r"""Image detail level for vision models"""


class ChatContentImageTypedDict(TypedDict):
    r"""Image content part for vision models"""

    type: ChatContentImageType
    image_url: ChatContentImageImageURLTypedDict


class ChatContentImage(BaseModel):
    r"""Image content part for vision models"""

    type: ChatContentImageType

    image_url: ChatContentImageImageURL



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .annotationaddedevent import AnnotationAddedEvent, AnnotationAddedEventTypedDict
from .contentpartaddedevent import ContentPartAddedEvent, ContentPartAddedEventTypedDict
from .contentpartdoneevent import ContentPartDoneEvent, ContentPartDoneEventTypedDict
from .errorevent import ErrorEvent, ErrorEventTypedDict
from .functioncallargsdeltaevent import (
    FunctionCallArgsDeltaEvent,
    FunctionCallArgsDeltaEventTypedDict,
)
from .functioncallargsdoneevent import (
    FunctionCallArgsDoneEvent,
    FunctionCallArgsDoneEventTypedDict,
)
from .imagegencallcompletedevent import (
    ImageGenCallCompletedEvent,
    ImageGenCallCompletedEventTypedDict,
)
from .imagegencallgeneratingevent import (
    ImageGenCallGeneratingEvent,
    ImageGenCallGeneratingEventTypedDict,
)
from .imagegencallinprogressevent import (
    ImageGenCallInProgressEvent,
    ImageGenCallInProgressEventTypedDict,
)
from .imagegencallpartialimageevent import (
    ImageGenCallPartialImageEvent,
    ImageGenCallPartialImageEventTypedDict,
)
from .openresponsesresult import OpenResponsesResult, OpenResponsesResultTypedDict
from .outputitems import OutputItems, OutputItemsTypedDict
from .reasoningdeltaevent import ReasoningDeltaEvent, ReasoningDeltaEventTypedDict
from .reasoningdoneevent import ReasoningDoneEvent, ReasoningDoneEventTypedDict
from .reasoningsummarypartaddedevent import (
    ReasoningSummaryPartAddedEvent,
    ReasoningSummaryPartAddedEventTypedDict,
)
from .reasoningsummarypartdoneevent import (
    ReasoningSummaryPartDoneEvent,
    ReasoningSummaryPartDoneEventTypedDict,
)
from .reasoningsummarytextdeltaevent import (
    ReasoningSummaryTextDeltaEvent,
    ReasoningSummaryTextDeltaEventTypedDict,
)
from .reasoningsummarytextdoneevent import (
    ReasoningSummaryTextDoneEvent,
    ReasoningSummaryTextDoneEventTypedDict,
)
from .refusaldeltaevent import RefusalDeltaEvent, RefusalDeltaEventTypedDict
from .refusaldoneevent import RefusalDoneEvent, RefusalDoneEventTypedDict
from .textdeltaevent import TextDeltaEvent, TextDeltaEventTypedDict
from .textdoneevent import TextDoneEvent, TextDoneEventTypedDict
from .websearchcallcompletedevent import (
    WebSearchCallCompletedEvent,
    WebSearchCallCompletedEventTypedDict,
)
from .websearchcallinprogressevent import (
    WebSearchCallInProgressEvent,
    WebSearchCallInProgressEventTypedDict,
)
from .websearchcallsearchingevent import (
    WebSearchCallSearchingEvent,
    WebSearchCallSearchingEventTypedDict,
)
from openrouter.types import BaseModel
from openrouter.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Literal, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


TypeResponseOutputItemDone = Literal["response.output_item.done",]


class StreamEventsResponseOutputItemDoneTypedDict(TypedDict):
    r"""Event emitted when an output item is complete"""

    type: TypeResponseOutputItemDone
    output_index: float
    item: OutputItemsTypedDict
    r"""An output item from the response"""
    sequence_number: float


class StreamEventsResponseOutputItemDone(BaseModel):
    r"""Event emitted when an output item is complete"""

    type: TypeResponseOutputItemDone

    output_index: float

    item: OutputItems
    r"""An output item from the response"""

    sequence_number: float


TypeResponseOutputItemAdded = Literal["response.output_item.added",]


class StreamEventsResponseOutputItemAddedTypedDict(TypedDict):
    r"""Event emitted when a new output item is added to the response"""

    type: TypeResponseOutputItemAdded
    output_index: float
    item: OutputItemsTypedDict
    r"""An output item from the response"""
    sequence_number: float


class StreamEventsResponseOutputItemAdded(BaseModel):
    r"""Event emitted when a new output item is added to the response"""

    type: TypeResponseOutputItemAdded

    output_index: float

    item: OutputItems
    r"""An output item from the response"""

    sequence_number: float


TypeResponseFailed = Literal["response.failed",]


class StreamEventsResponseFailedTypedDict(TypedDict):
    r"""Event emitted when a response has failed"""

    type: TypeResponseFailed
    response: OpenResponsesResultTypedDict
    r"""Complete non-streaming response from the Responses API"""
    sequence_number: float


class StreamEventsResponseFailed(BaseModel):
    r"""Event emitted when a response has failed"""

    type: TypeResponseFailed

    response: OpenResponsesResult
    r"""Complete non-streaming response from the Responses API"""

    sequence_number: float


TypeResponseIncomplete = Literal["response.incomplete",]


class StreamEventsResponseIncompleteTypedDict(TypedDict):
    r"""Event emitted when a response is incomplete"""

    type: TypeResponseIncomplete
    response: OpenResponsesResultTypedDict
    r"""Complete non-streaming response from the Responses API"""
    sequence_number: float


class StreamEventsResponseIncomplete(BaseModel):
    r"""Event emitted when a response is incomplete"""

    type: TypeResponseIncomplete

    response: OpenResponsesResult
    r"""Complete non-streaming response from the Responses API"""

    sequence_number: float


TypeResponseCompleted = Literal["response.completed",]


class StreamEventsResponseCompletedTypedDict(TypedDict):
    r"""Event emitted when a response has completed successfully"""

    type: TypeResponseCompleted
    response: OpenResponsesResultTypedDict
    r"""Complete non-streaming response from the Responses API"""
    sequence_number: float


class StreamEventsResponseCompleted(BaseModel):
    r"""Event emitted when a response has completed successfully"""

    type: TypeResponseCompleted

    response: OpenResponsesResult
    r"""Complete non-streaming response from the Responses API"""

    sequence_number: float


TypeResponseInProgress = Literal["response.in_progress",]


class StreamEventsResponseInProgressTypedDict(TypedDict):
    r"""Event emitted when a response is in progress"""

    type: TypeResponseInProgress
    response: OpenResponsesResultTypedDict
    r"""Complete non-streaming response from the Responses API"""
    sequence_number: float


class StreamEventsResponseInProgress(BaseModel):
    r"""Event emitted when a response is in progress"""

    type: TypeResponseInProgress

    response: OpenResponsesResult
    r"""Complete non-streaming response from the Responses API"""

    sequence_number: float


TypeResponseCreated = Literal["response.created",]


class StreamEventsResponseCreatedTypedDict(TypedDict):
    r"""Event emitted when a response is created"""

    type: TypeResponseCreated
    response: OpenResponsesResultTypedDict
    r"""Complete non-streaming response from the Responses API"""
    sequence_number: float


class StreamEventsResponseCreated(BaseModel):
    r"""Event emitted when a response is created"""

    type: TypeResponseCreated

    response: OpenResponsesResult
    r"""Complete non-streaming response from the Responses API"""

    sequence_number: float


StreamEventsTypedDict = TypeAliasType(
    "StreamEventsTypedDict",
    Union[
        StreamEventsResponseCreatedTypedDict,
        StreamEventsResponseInProgressTypedDict,
        StreamEventsResponseCompletedTypedDict,
        StreamEventsResponseIncompleteTypedDict,
        StreamEventsResponseFailedTypedDict,
        StreamEventsResponseOutputItemAddedTypedDict,
        StreamEventsResponseOutputItemDoneTypedDict,
        WebSearchCallCompletedEventTypedDict,
        WebSearchCallSearchingEventTypedDict,
        WebSearchCallInProgressEventTypedDict,
        ImageGenCallCompletedEventTypedDict,
        ImageGenCallGeneratingEventTypedDict,
        ImageGenCallInProgressEventTypedDict,
        FunctionCallArgsDeltaEventTypedDict,
        ErrorEventTypedDict,
        ReasoningSummaryTextDeltaEventTypedDict,
        ReasoningSummaryTextDoneEventTypedDict,
        ReasoningDeltaEventTypedDict,
        ReasoningDoneEventTypedDict,
        ReasoningSummaryPartAddedEventTypedDict,
        ReasoningSummaryPartDoneEventTypedDict,
        ContentPartAddedEventTypedDict,
        FunctionCallArgsDoneEventTypedDict,
        RefusalDoneEventTypedDict,
        RefusalDeltaEventTypedDict,
        ImageGenCallPartialImageEventTypedDict,
        ContentPartDoneEventTypedDict,
        TextDeltaEventTypedDict,
        TextDoneEventTypedDict,
        AnnotationAddedEventTypedDict,
    ],
)
r"""Union of all possible event types emitted during response streaming"""


StreamEvents = Annotated[
    Union[
        Annotated[StreamEventsResponseCreated, Tag("response.created")],
        Annotated[StreamEventsResponseInProgress, Tag("response.in_progress")],
        Annotated[StreamEventsResponseCompleted, Tag("response.completed")],
        Annotated[StreamEventsResponseIncomplete, Tag("response.incomplete")],
        Annotated[StreamEventsResponseFailed, Tag("response.failed")],
        Annotated[ErrorEvent, Tag("error")],
        Annotated[
            StreamEventsResponseOutputItemAdded, Tag("response.output_item.added")
        ],
        Annotated[StreamEventsResponseOutputItemDone, Tag("response.output_item.done")],
        Annotated[ContentPartAddedEvent, Tag("response.content_part.added")],
        Annotated[ContentPartDoneEvent, Tag("response.content_part.done")],
        Annotated[TextDeltaEvent, Tag("response.output_text.delta")],
        Annotated[TextDoneEvent, Tag("response.output_text.done")],
        Annotated[RefusalDeltaEvent, Tag("response.refusal.delta")],
        Annotated[RefusalDoneEvent, Tag("response.refusal.done")],
        Annotated[AnnotationAddedEvent, Tag("response.output_text.annotation.added")],
        Annotated[
            FunctionCallArgsDeltaEvent, Tag("response.function_call_arguments.delta")
        ],
        Annotated[
            FunctionCallArgsDoneEvent, Tag("response.function_call_arguments.done")
        ],
        Annotated[ReasoningDeltaEvent, Tag("response.reasoning_text.delta")],
        Annotated[ReasoningDoneEvent, Tag("response.reasoning_text.done")],
        Annotated[
            ReasoningSummaryPartAddedEvent, Tag("response.reasoning_summary_part.added")
        ],
        Annotated[
            ReasoningSummaryPartDoneEvent, Tag("response.reasoning_summary_part.done")
        ],
        Annotated[
            ReasoningSummaryTextDeltaEvent, Tag("response.reasoning_summary_text.delta")
        ],
        Annotated[
            ReasoningSummaryTextDoneEvent, Tag("response.reasoning_summary_text.done")
        ],
        Annotated[
            ImageGenCallInProgressEvent,
            Tag("response.image_generation_call.in_progress"),
        ],
        Annotated[
            ImageGenCallGeneratingEvent,
            Tag("response.image_generation_call.generating"),
        ],
        Annotated[
            ImageGenCallPartialImageEvent,
            Tag("response.image_generation_call.partial_image"),
        ],
        Annotated[
            ImageGenCallCompletedEvent, Tag("response.image_generation_call.completed")
        ],
        Annotated[
            WebSearchCallInProgressEvent, Tag("response.web_search_call.in_progress")
        ],
        Annotated[
            WebSearchCallSearchingEvent, Tag("response.web_search_call.searching")
        ],
        Annotated[
            WebSearchCallCompletedEvent, Tag("response.web_search_call.completed")
        ],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]
r"""Union of all possible event types emitted during response streaming"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .reasoningdetailencrypted import (
    ReasoningDetailEncrypted,
    ReasoningDetailEncryptedTypedDict,
)
from .reasoningdetailsummary import (
    ReasoningDetailSummary,
    ReasoningDetailSummaryTypedDict,
)
from .reasoningdetailtext import ReasoningDetailText, ReasoningDetailTextTypedDict
from openrouter.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


ReasoningDetailUnionTypedDict = TypeAliasType(
    "ReasoningDetailUnionTypedDict",
    Union[
        ReasoningDetailSummaryTypedDict,
        ReasoningDetailEncryptedTypedDict,
        ReasoningDetailTextTypedDict,
    ],
)
r"""Reasoning detail union schema"""


ReasoningDetailUnion = Annotated[
    Union[
        Annotated[ReasoningDetailSummary, Tag("reasoning.summary")],
        Annotated[ReasoningDetailEncrypted, Tag("reasoning.encrypted")],
        Annotated[ReasoningDetailText, Tag("reasoning.text")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]
r"""Reasoning detail union schema"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatcontentcachecontrol import (
    ChatContentCacheControl,
    ChatContentCacheControlTypedDict,
)
from openrouter.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


ChatContentTextType = Literal["text",]


class ChatContentTextTypedDict(TypedDict):
    r"""Text content part"""

    type: ChatContentTextType
    text: str
    cache_control: NotRequired[ChatContentCacheControlTypedDict]
    r"""Cache control for the content part"""


class ChatContentText(BaseModel):
    r"""Text content part"""

    type: ChatContentTextType

    text: str

    cache_control: Optional[ChatContentCacheControl] = None
    r"""Cache control for the content part"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatcontentitems import ChatContentItems, ChatContentItemsTypedDict
from openrouter.types import BaseModel
from typing import List, Literal, Union
from typing_extensions import TypeAliasType, TypedDict


ChatToolMessageRole = Literal["tool",]


ChatToolMessageContentTypedDict = TypeAliasType(
    "ChatToolMessageContentTypedDict", Union[str, List[ChatContentItemsTypedDict]]
)
r"""Tool response content"""


ChatToolMessageContent = TypeAliasType(
    "ChatToolMessageContent", Union[str, List[ChatContentItems]]
)
r"""Tool response content"""


class ChatToolMessageTypedDict(TypedDict):
    r"""Tool response message"""

    role: ChatToolMessageRole
    content: ChatToolMessageContentTypedDict
    r"""Tool response content"""
    tool_call_id: str
    r"""ID of the assistant message tool call this message responds to"""


class ChatToolMessage(BaseModel):
    r"""Tool response message"""

    role: ChatToolMessageRole

    content: ChatToolMessageContent
    r"""Tool response content"""

    tool_call_id: str
    r"""ID of the assistant message tool call this message responds to"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, Nullable, UNSET_SENTINEL
from openrouter.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    QueryParamMetadata,
)
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ListGuardrailMemberAssignmentsGlobalsTypedDict(TypedDict):
    http_referer: NotRequired[str]
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """
    x_open_router_title: NotRequired[str]
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """
    x_open_router_categories: NotRequired[str]
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class ListGuardrailMemberAssignmentsGlobals(BaseModel):
    http_referer: Annotated[
        Optional[str],
        pydantic.Field(alias="HTTP-Referer"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """

    x_open_router_title: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Title"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """

    x_open_router_categories: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Categories"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class ListGuardrailMemberAssignmentsRequestTypedDict(TypedDict):
    id: str
    r"""The unique identifier of the guardrail"""
    http_referer: NotRequired[str]
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """
    x_open_router_title: NotRequired[str]
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """
    x_open_router_categories: NotRequired[str]
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """
    offset: NotRequired[str]
    r"""Number of records to skip for pagination"""
    limit: NotRequired[str]
    r"""Maximum number of records to return (max 100)"""


class ListGuardrailMemberAssignmentsRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The unique identifier of the guardrail"""

    http_referer: Annotated[
        Optional[str],
        pydantic.Field(alias="HTTP-Referer"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """

    x_open_router_title: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Title"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """

    x_open_router_categories: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Categories"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """

    offset: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Number of records to skip for pagination"""

    limit: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Maximum number of records to return (max 100)"""


class ListGuardrailMemberAssignmentsDataTypedDict(TypedDict):
    id: str
    r"""Unique identifier for the assignment"""
    user_id: str
    r"""Clerk user ID of the assigned member"""
    organization_id: str
    r"""Organization ID"""
    guardrail_id: str
    r"""ID of the guardrail"""
    assigned_by: Nullable[str]
    r"""User ID of who made the assignment"""
    created_at: str
    r"""ISO 8601 timestamp of when the assignment was created"""


class ListGuardrailMemberAssignmentsData(BaseModel):
    id: str
    r"""Unique identifier for the assignment"""

    user_id: str
    r"""Clerk user ID of the assigned member"""

    organization_id: str
    r"""Organization ID"""

    guardrail_id: str
    r"""ID of the guardrail"""

    assigned_by: Nullable[str]
    r"""User ID of who made the assignment"""

    created_at: str
    r"""ISO 8601 timestamp of when the assignment was created"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["assigned_by"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ListGuardrailMemberAssignmentsResponseTypedDict(TypedDict):
    r"""List of member assignments"""

    data: List[ListGuardrailMemberAssignmentsDataTypedDict]
    r"""List of member assignments"""
    total_count: float
    r"""Total number of member assignments"""


class ListGuardrailMemberAssignmentsResponse(BaseModel):
    r"""List of member assignments"""

    data: List[ListGuardrailMemberAssignmentsData]
    r"""List of member assignments"""

    total_count: float
    r"""Total number of member assignments"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


RefusalDeltaEventType = Literal["response.refusal.delta",]


class RefusalDeltaEventTypedDict(TypedDict):
    r"""Event emitted when a refusal delta is streamed"""

    type: RefusalDeltaEventType
    output_index: float
    item_id: str
    content_index: float
    delta: str
    sequence_number: float


class RefusalDeltaEvent(BaseModel):
    r"""Event emitted when a refusal delta is streamed"""

    type: RefusalDeltaEventType

    output_index: float

    item_id: str

    content_index: float

    delta: str

    sequence_number: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


RefusalDoneEventType = Literal["response.refusal.done",]


class RefusalDoneEventTypedDict(TypedDict):
    r"""Event emitted when refusal streaming is complete"""

    type: RefusalDoneEventType
    output_index: float
    item_id: str
    content_index: float
    refusal: str
    sequence_number: float


class RefusalDoneEvent(BaseModel):
    r"""Event emitted when refusal streaming is complete"""

    type: RefusalDoneEventType

    output_index: float

    item_id: str

    content_index: float

    refusal: str

    sequence_number: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


OutputFunctionCallItemType = Literal["function_call",]


OutputFunctionCallItemStatusInProgress = Literal["in_progress",]


OutputFunctionCallItemStatusIncomplete = Literal["incomplete",]


OutputFunctionCallItemStatusCompleted = Literal["completed",]


OutputFunctionCallItemStatusUnionTypedDict = TypeAliasType(
    "OutputFunctionCallItemStatusUnionTypedDict",
    Union[
        OutputFunctionCallItemStatusCompleted,
        OutputFunctionCallItemStatusIncomplete,
        OutputFunctionCallItemStatusInProgress,
    ],
)


OutputFunctionCallItemStatusUnion = TypeAliasType(
    "OutputFunctionCallItemStatusUnion",
    Union[
        OutputFunctionCallItemStatusCompleted,
        OutputFunctionCallItemStatusIncomplete,
        OutputFunctionCallItemStatusInProgress,
    ],
)


class OutputFunctionCallItemTypedDict(TypedDict):
    type: OutputFunctionCallItemType
    name: str
    arguments: str
    call_id: str
    id: NotRequired[str]
    status: NotRequired[OutputFunctionCallItemStatusUnionTypedDict]


class OutputFunctionCallItem(BaseModel):
    type: OutputFunctionCallItemType

    name: str

    arguments: str

    call_id: str

    id: Optional[str] = None

    status: Optional[OutputFunctionCallItemStatusUnion] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from enum import Enum
from openrouter import components, errors, operations, utils
from openrouter._hooks import HookContext
from openrouter.types import OptionalNullable, UNSET
from openrouter.utils import eventstreaming, get_security_from_env
from openrouter.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Literal, Mapping, Optional, Union, overload


class SendAcceptEnum(str, Enum):
    APPLICATION_JSON = "application/json"
    TEXT_EVENT_STREAM = "text/event-stream"


class Chat(BaseSDK):
    @overload
    def send(
        self,
        *,
        messages: Union[
            List[components.ChatMessages], List[components.ChatMessagesTypedDict]
        ],
        http_referer: Optional[str] = None,
        x_open_router_title: Optional[str] = None,
        x_open_router_categories: Optional[str] = None,
        provider: OptionalNullable[
            Union[
                components.ChatRequestProvider, components.ChatRequestProviderTypedDict
            ]
        ] = UNSET,
        plugins: Optional[
            Union[
                List[components.ChatRequestPluginUnion],
                List[components.ChatRequestPluginUnionTypedDict],
            ]
        ] = None,
        user: Optional[str] = None,
        session_id: Optional[str] = None,
        trace: Optional[
            Union[components.ChatRequestTrace, components.ChatRequestTraceTypedDict]
        ] = None,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[Dict[str, float]] = UNSET,
        logprobs: OptionalNullable[bool] = UNSET,
        top_logprobs: OptionalNullable[float] = UNSET,
        max_completion_tokens: OptionalNullable[float] = UNSET,
        max_tokens: OptionalNullable[float] = UNSET,
        metadata: Optional[Dict[str, str]] = None,
        presence_penalty: OptionalNullable[float] = UNSET,
        reasoning: Optional[
            Union[components.Reasoning, components.ReasoningTypedDict]
        ] = None,
        response_format: Optional[
            Union[components.ResponseFormat, components.ResponseFormatTypedDict]
        ] = None,
        seed: OptionalNullable[int] = UNSET,
        stop: OptionalNullable[
            Union[components.Stop, components.StopTypedDict]
        ] = UNSET,
        stream: Union[Literal[False], None] = None,
        stream_options: OptionalNullable[
            Union[components.ChatStreamOptions, components.ChatStreamOptionsTypedDict]
        ] = UNSET,
        temperature: OptionalNullable[float] = 1,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        tool_choice: Optional[
            Union[components.ChatToolChoice, components.ChatToolChoiceTypedDict]
        ] = None,
        tools: Optional[
            Union[
                List[components.ChatFunctionTool],
                List[components.ChatFunctionToolTypedDict],
            ]
        ] = None,
        top_p: OptionalNullable[float] = 1,
        debug: Optional[
            Union[components.ChatDebugOptions, components.ChatDebugOptionsTypedDict]
        ] = None,
        image_config: Optional[
            Union[
                Dict[str, components.ChatRequestImageConfig],
                Dict[str, components.ChatRequestImageConfigTypedDict],
            ]
        ] = None,
        modalities: Optional[List[components.Modality]] = None,
        cache_control: Optional[
            Union[components.CacheControl, components.CacheControlTypedDict]
        ] = None,
        service_tier: OptionalNullable[components.ChatRequestServiceTier] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> components.ChatResult:
        r"""Create a chat completion

        Sends a request for a model response for the given chat conversation. Supports both streaming and non-streaming modes.

        :param messages: List of messages for the conversation
        :param http_referer: The app identifier should be your app's URL and is used as the primary identifier for rankings.
            This is used to track API usage per application.

        :param x_open_router_title: The app display name allows you to customize how your app appears in OpenRouter's dashboard.

        :param x_open_router_categories: Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param user: Unique user identifier
        :param session_id: A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 256 characters.
        :param trace: Metadata for observability and tracing. Known keys (trace_id, trace_name, span_name, generation_name, parent_span_id) have special handling. Additional keys are passed through as custom metadata to configured broadcast destinations.
        :param model: Model to use for completion
        :param models: Models to use for completion
        :param frequency_penalty: Frequency penalty (-2.0 to 2.0)
        :param logit_bias: Token logit bias adjustments
        :param logprobs: Return log probabilities
        :param top_logprobs: Number of top log probabilities to return (0-20)
        :param max_completion_tokens: Maximum tokens in completion
        :param max_tokens: Maximum tokens (deprecated, use max_completion_tokens). Note: some providers enforce a minimum of 16.
        :param metadata: Key-value pairs for additional object information (max 16 pairs, 64 char keys, 512 char values)
        :param presence_penalty: Presence penalty (-2.0 to 2.0)
        :param reasoning: Configuration options for reasoning models
        :param response_format: Response format configuration
        :param seed: Random seed for deterministic outputs
        :param stop: Stop sequences (up to 4)
        :param stream: Enable streaming response
        :param stream_options: Streaming configuration options
        :param temperature: Sampling temperature (0-2)
        :param parallel_tool_calls:
        :param tool_choice: Tool choice configuration
        :param tools: Available tools for function calling
        :param top_p: Nucleus sampling parameter (0-1)
        :param debug: Debug options for inspecting request transformations (streaming only)
        :param image_config: Provider-specific image configuration options. Keys and values vary by model/provider. See https://openrouter.ai/docs/guides/overview/multimodal/image-generation for more details.
        :param modalities: Output modalities for the response. Supported values are \"text\", \"image\", and \"audio\".
        :param cache_control: Enable automatic prompt caching. When set, the system automatically applies cache breakpoints to the last cacheable block in the request. Currently supported for Anthropic Claude models.
        :param service_tier: The service tier to use for processing this request.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    @overload
    def send(
        self,
        *,
        messages: Union[
            List[components.ChatMessages], List[components.ChatMessagesTypedDict]
        ],
        http_referer: Optional[str] = None,
        x_open_router_title: Optional[str] = None,
        x_open_router_categories: Optional[str] = None,
        provider: OptionalNullable[
            Union[
                components.ChatRequestProvider, components.ChatRequestProviderTypedDict
            ]
        ] = UNSET,
        plugins: Optional[
            Union[
                List[components.ChatRequestPluginUnion],
                List[components.ChatRequestPluginUnionTypedDict],
            ]
        ] = None,
        user: Optional[str] = None,
        session_id: Optional[str] = None,
        trace: Optional[
            Union[components.ChatRequestTrace, components.ChatRequestTraceTypedDict]
        ] = None,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[Dict[str, float]] = UNSET,
        logprobs: OptionalNullable[bool] = UNSET,
        top_logprobs: OptionalNullable[float] = UNSET,
        max_completion_tokens: OptionalNullable[float] = UNSET,
        max_tokens: OptionalNullable[float] = UNSET,
        metadata: Optional[Dict[str, str]] = None,
        presence_penalty: OptionalNullable[float] = UNSET,
        reasoning: Optional[
            Union[components.Reasoning, components.ReasoningTypedDict]
        ] = None,
        response_format: Optional[
            Union[components.ResponseFormat, components.ResponseFormatTypedDict]
        ] = None,
        seed: OptionalNullable[int] = UNSET,
        stop: OptionalNullable[
            Union[components.Stop, components.StopTypedDict]
        ] = UNSET,
        stream: Literal[True],
        stream_options: OptionalNullable[
            Union[components.ChatStreamOptions, components.ChatStreamOptionsTypedDict]
        ] = UNSET,
        temperature: OptionalNullable[float] = 1,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        tool_choice: Optional[
            Union[components.ChatToolChoice, components.ChatToolChoiceTypedDict]
        ] = None,
        tools: Optional[
            Union[
                List[components.ChatFunctionTool],
                List[components.ChatFunctionToolTypedDict],
            ]
        ] = None,
        top_p: OptionalNullable[float] = 1,
        debug: Optional[
            Union[components.ChatDebugOptions, components.ChatDebugOptionsTypedDict]
        ] = None,
        image_config: Optional[
            Union[
                Dict[str, components.ChatRequestImageConfig],
                Dict[str, components.ChatRequestImageConfigTypedDict],
            ]
        ] = None,
        modalities: Optional[List[components.Modality]] = None,
        cache_control: Optional[
            Union[components.CacheControl, components.CacheControlTypedDict]
        ] = None,
        service_tier: OptionalNullable[components.ChatRequestServiceTier] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStream[components.ChatStreamChunk]:
        r"""Create a chat completion

        Sends a request for a model response for the given chat conversation. Supports both streaming and non-streaming modes.

        :param messages: List of messages for the conversation
        :param http_referer: The app identifier should be your app's URL and is used as the primary identifier for rankings.
            This is used to track API usage per application.

        :param x_open_router_title: The app display name allows you to customize how your app appears in OpenRouter's dashboard.

        :param x_open_router_categories: Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param user: Unique user identifier
        :param session_id: A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 256 characters.
        :param trace: Metadata for observability and tracing. Known keys (trace_id, trace_name, span_name, generation_name, parent_span_id) have special handling. Additional keys are passed through as custom metadata to configured broadcast destinations.
        :param model: Model to use for completion
        :param models: Models to use for completion
        :param frequency_penalty: Frequency penalty (-2.0 to 2.0)
        :param logit_bias: Token logit bias adjustments
        :param logprobs: Return log probabilities
        :param top_logprobs: Number of top log probabilities to return (0-20)
        :param max_completion_tokens: Maximum tokens in completion
        :param max_tokens: Maximum tokens (deprecated, use max_completion_tokens). Note: some providers enforce a minimum of 16.
        :param metadata: Key-value pairs for additional object information (max 16 pairs, 64 char keys, 512 char values)
        :param presence_penalty: Presence penalty (-2.0 to 2.0)
        :param reasoning: Configuration options for reasoning models
        :param response_format: Response format configuration
        :param seed: Random seed for deterministic outputs
        :param stop: Stop sequences (up to 4)
        :param stream: Enable streaming response
        :param stream_options: Streaming configuration options
        :param temperature: Sampling temperature (0-2)
        :param parallel_tool_calls:
        :param tool_choice: Tool choice configuration
        :param tools: Available tools for function calling
        :param top_p: Nucleus sampling parameter (0-1)
        :param debug: Debug options for inspecting request transformations (streaming only)
        :param image_config: Provider-specific image configuration options. Keys and values vary by model/provider. See https://openrouter.ai/docs/guides/overview/multimodal/image-generation for more details.
        :param modalities: Output modalities for the response. Supported values are \"text\", \"image\", and \"audio\".
        :param cache_control: Enable automatic prompt caching. When set, the system automatically applies cache breakpoints to the last cacheable block in the request. Currently supported for Anthropic Claude models.
        :param service_tier: The service tier to use for processing this request.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    def send(
        self,
        *,
        messages: Union[
            List[components.ChatMessages], List[components.ChatMessagesTypedDict]
        ],
        http_referer: Optional[str] = None,
        x_open_router_title: Optional[str] = None,
        x_open_router_categories: Optional[str] = None,
        provider: OptionalNullable[
            Union[
                components.ChatRequestProvider, components.ChatRequestProviderTypedDict
            ]
        ] = UNSET,
        plugins: Optional[
            Union[
                List[components.ChatRequestPluginUnion],
                List[components.ChatRequestPluginUnionTypedDict],
            ]
        ] = None,
        user: Optional[str] = None,
        session_id: Optional[str] = None,
        trace: Optional[
            Union[components.ChatRequestTrace, components.ChatRequestTraceTypedDict]
        ] = None,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[Dict[str, float]] = UNSET,
        logprobs: OptionalNullable[bool] = UNSET,
        top_logprobs: OptionalNullable[float] = UNSET,
        max_completion_tokens: OptionalNullable[float] = UNSET,
        max_tokens: OptionalNullable[float] = UNSET,
        metadata: Optional[Dict[str, str]] = None,
        presence_penalty: OptionalNullable[float] = UNSET,
        reasoning: Optional[
            Union[components.Reasoning, components.ReasoningTypedDict]
        ] = None,
        response_format: Optional[
            Union[components.ResponseFormat, components.ResponseFormatTypedDict]
        ] = None,
        seed: OptionalNullable[int] = UNSET,
        stop: OptionalNullable[
            Union[components.Stop, components.StopTypedDict]
        ] = UNSET,
        stream: Optional[bool] = False,
        stream_options: OptionalNullable[
            Union[components.ChatStreamOptions, components.ChatStreamOptionsTypedDict]
        ] = UNSET,
        temperature: OptionalNullable[float] = 1,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        tool_choice: Optional[
            Union[components.ChatToolChoice, components.ChatToolChoiceTypedDict]
        ] = None,
        tools: Optional[
            Union[
                List[components.ChatFunctionTool],
                List[components.ChatFunctionToolTypedDict],
            ]
        ] = None,
        top_p: OptionalNullable[float] = 1,
        debug: Optional[
            Union[components.ChatDebugOptions, components.ChatDebugOptionsTypedDict]
        ] = None,
        image_config: Optional[
            Union[
                Dict[str, components.ChatRequestImageConfig],
                Dict[str, components.ChatRequestImageConfigTypedDict],
            ]
        ] = None,
        modalities: Optional[List[components.Modality]] = None,
        cache_control: Optional[
            Union[components.CacheControl, components.CacheControlTypedDict]
        ] = None,
        service_tier: OptionalNullable[components.ChatRequestServiceTier] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.SendChatCompletionRequestResponse:
        r"""Create a chat completion

        Sends a request for a model response for the given chat conversation. Supports both streaming and non-streaming modes.

        :param messages: List of messages for the conversation
        :param http_referer: The app identifier should be your app's URL and is used as the primary identifier for rankings.
            This is used to track API usage per application.

        :param x_open_router_title: The app display name allows you to customize how your app appears in OpenRouter's dashboard.

        :param x_open_router_categories: Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param user: Unique user identifier
        :param session_id: A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 256 characters.
        :param trace: Metadata for observability and tracing. Known keys (trace_id, trace_name, span_name, generation_name, parent_span_id) have special handling. Additional keys are passed through as custom metadata to configured broadcast destinations.
        :param model: Model to use for completion
        :param models: Models to use for completion
        :param frequency_penalty: Frequency penalty (-2.0 to 2.0)
        :param logit_bias: Token logit bias adjustments
        :param logprobs: Return log probabilities
        :param top_logprobs: Number of top log probabilities to return (0-20)
        :param max_completion_tokens: Maximum tokens in completion
        :param max_tokens: Maximum tokens (deprecated, use max_completion_tokens). Note: some providers enforce a minimum of 16.
        :param metadata: Key-value pairs for additional object information (max 16 pairs, 64 char keys, 512 char values)
        :param presence_penalty: Presence penalty (-2.0 to 2.0)
        :param reasoning: Configuration options for reasoning models
        :param response_format: Response format configuration
        :param seed: Random seed for deterministic outputs
        :param stop: Stop sequences (up to 4)
        :param stream: Enable streaming response
        :param stream_options: Streaming configuration options
        :param temperature: Sampling temperature (0-2)
        :param parallel_tool_calls:
        :param tool_choice: Tool choice configuration
        :param tools: Available tools for function calling
        :param top_p: Nucleus sampling parameter (0-1)
        :param debug: Debug options for inspecting request transformations (streaming only)
        :param image_config: Provider-specific image configuration options. Keys and values vary by model/provider. See https://openrouter.ai/docs/guides/overview/multimodal/image-generation for more details.
        :param modalities: Output modalities for the response. Supported values are \"text\", \"image\", and \"audio\".
        :param cache_control: Enable automatic prompt caching. When set, the system automatically applies cache breakpoints to the last cacheable block in the request. Currently supported for Anthropic Claude models.
        :param service_tier: The service tier to use for processing this request.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.SendChatCompletionRequestRequest(
            http_referer=http_referer,
            x_open_router_title=x_open_router_title,
            x_open_router_categories=x_open_router_categories,
            chat_request=components.ChatRequest(
                provider=utils.get_pydantic_model(
                    provider, OptionalNullable[components.ChatRequestProvider]
                ),
                plugins=utils.get_pydantic_model(
                    plugins, Optional[List[components.ChatRequestPluginUnion]]
                ),
                user=user,
                session_id=session_id,
                trace=utils.get_pydantic_model(
                    trace, Optional[components.ChatRequestTrace]
                ),
                messages=utils.get_pydantic_model(
                    messages, List[components.ChatMessages]
                ),
                model=model,
                models=models,
                frequency_penalty=frequency_penalty,
                logit_bias=logit_bias,
                logprobs=logprobs,
                top_logprobs=top_logprobs,
                max_completion_tokens=max_completion_tokens,
                max_tokens=max_tokens,
                metadata=metadata,
                presence_penalty=presence_penalty,
                reasoning=utils.get_pydantic_model(
                    reasoning, Optional[components.Reasoning]
                ),
                response_format=utils.get_pydantic_model(
                    response_format, Optional[components.ResponseFormat]
                ),
                seed=seed,
                stop=stop,
                stream=stream,
                stream_options=utils.get_pydantic_model(
                    stream_options, OptionalNullable[components.ChatStreamOptions]
                ),
                temperature=temperature,
                parallel_tool_calls=parallel_tool_calls,
                tool_choice=utils.get_pydantic_model(
                    tool_choice, Optional[components.ChatToolChoice]
                ),
                tools=utils.get_pydantic_model(
                    tools, Optional[List[components.ChatFunctionTool]]
                ),
                top_p=top_p,
                debug=utils.get_pydantic_model(
                    debug, Optional[components.ChatDebugOptions]
                ),
                image_config=image_config,
                modalities=modalities,
                cache_control=utils.get_pydantic_model(
                    cache_control, Optional[components.CacheControl]
                ),
                service_tier=service_tier,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/chat/completions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream" if stream else "application/json",
            http_headers=http_headers,
            _globals=operations.SendChatCompletionRequestGlobals(
                http_referer=self.sdk_configuration.globals.http_referer,
                x_open_router_title=self.sdk_configuration.globals.x_open_router_title,
                x_open_router_categories=self.sdk_configuration.globals.x_open_router_categories,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.chat_request, False, False, "json", components.ChatRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="sendChatCompletionRequest",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "404",
                "408",
                "413",
                "422",
                "429",
                "4XX",
                "500",
                "502",
                "503",
                "524",
                "529",
                "5XX",
            ],
            stream=True,
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            return unmarshal_json_response(
                components.ChatResult, http_res, http_res_text
            )
        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStream(
                http_res,
                lambda raw: utils.unmarshal_json(
                    raw, operations.SendChatCompletionRequestResponseBody
                ).data,
                sentinel="[DONE]",
                client_ref=self,
            )
        if utils.match_response(http_res, "400", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res, http_res_text
            )
            raise errors.BadRequestResponseError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "401", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res, http_res_text
            )
            raise errors.UnauthorizedResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "402", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.PaymentRequiredResponseErrorData, http_res, http_res_text
            )
            raise errors.PaymentRequiredResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "404", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.NotFoundResponseErrorData, http_res, http_res_text
            )
            raise errors.NotFoundResponseError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "408", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.RequestTimeoutResponseErrorData, http_res, http_res_text
            )
            raise errors.RequestTimeoutResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "413", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.PayloadTooLargeResponseErrorData, http_res, http_res_text
            )
            raise errors.PayloadTooLargeResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "422", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityResponseErrorData, http_res, http_res_text
            )
            raise errors.UnprocessableEntityResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "429", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.TooManyRequestsResponseErrorData, http_res, http_res_text
            )
            raise errors.TooManyRequestsResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "500", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res, http_res_text
            )
            raise errors.InternalServerResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "502", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.BadGatewayResponseErrorData, http_res, http_res_text
            )
            raise errors.BadGatewayResponseError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "503", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableResponseErrorData, http_res, http_res_text
            )
            raise errors.ServiceUnavailableResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "524", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.EdgeNetworkTimeoutResponseErrorData, http_res, http_res_text
            )
            raise errors.EdgeNetworkTimeoutResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "529", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.ProviderOverloadedResponseErrorData, http_res, http_res_text
            )
            raise errors.ProviderOverloadedResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        http_res_text = utils.stream_to_text(http_res)
        raise errors.OpenRouterDefaultError(
            "Unexpected response received", http_res, http_res_text
        )

    @overload
    async def send_async(
        self,
        *,
        messages: Union[
            List[components.ChatMessages], List[components.ChatMessagesTypedDict]
        ],
        http_referer: Optional[str] = None,
        x_open_router_title: Optional[str] = None,
        x_open_router_categories: Optional[str] = None,
        provider: OptionalNullable[
            Union[
                components.ChatRequestProvider, components.ChatRequestProviderTypedDict
            ]
        ] = UNSET,
        plugins: Optional[
            Union[
                List[components.ChatRequestPluginUnion],
                List[components.ChatRequestPluginUnionTypedDict],
            ]
        ] = None,
        user: Optional[str] = None,
        session_id: Optional[str] = None,
        trace: Optional[
            Union[components.ChatRequestTrace, components.ChatRequestTraceTypedDict]
        ] = None,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[Dict[str, float]] = UNSET,
        logprobs: OptionalNullable[bool] = UNSET,
        top_logprobs: OptionalNullable[float] = UNSET,
        max_completion_tokens: OptionalNullable[float] = UNSET,
        max_tokens: OptionalNullable[float] = UNSET,
        metadata: Optional[Dict[str, str]] = None,
        presence_penalty: OptionalNullable[float] = UNSET,
        reasoning: Optional[
            Union[components.Reasoning, components.ReasoningTypedDict]
        ] = None,
        response_format: Optional[
            Union[components.ResponseFormat, components.ResponseFormatTypedDict]
        ] = None,
        seed: OptionalNullable[int] = UNSET,
        stop: OptionalNullable[
            Union[components.Stop, components.StopTypedDict]
        ] = UNSET,
        stream: Union[Literal[False], None] = None,
        stream_options: OptionalNullable[
            Union[components.ChatStreamOptions, components.ChatStreamOptionsTypedDict]
        ] = UNSET,
        temperature: OptionalNullable[float] = 1,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        tool_choice: Optional[
            Union[components.ChatToolChoice, components.ChatToolChoiceTypedDict]
        ] = None,
        tools: Optional[
            Union[
                List[components.ChatFunctionTool],
                List[components.ChatFunctionToolTypedDict],
            ]
        ] = None,
        top_p: OptionalNullable[float] = 1,
        debug: Optional[
            Union[components.ChatDebugOptions, components.ChatDebugOptionsTypedDict]
        ] = None,
        image_config: Optional[
            Union[
                Dict[str, components.ChatRequestImageConfig],
                Dict[str, components.ChatRequestImageConfigTypedDict],
            ]
        ] = None,
        modalities: Optional[List[components.Modality]] = None,
        cache_control: Optional[
            Union[components.CacheControl, components.CacheControlTypedDict]
        ] = None,
        service_tier: OptionalNullable[components.ChatRequestServiceTier] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> components.ChatResult:
        r"""Create a chat completion

        Sends a request for a model response for the given chat conversation. Supports both streaming and non-streaming modes.

        :param messages: List of messages for the conversation
        :param http_referer: The app identifier should be your app's URL and is used as the primary identifier for rankings.
            This is used to track API usage per application.

        :param x_open_router_title: The app display name allows you to customize how your app appears in OpenRouter's dashboard.

        :param x_open_router_categories: Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param user: Unique user identifier
        :param session_id: A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 256 characters.
        :param trace: Metadata for observability and tracing. Known keys (trace_id, trace_name, span_name, generation_name, parent_span_id) have special handling. Additional keys are passed through as custom metadata to configured broadcast destinations.
        :param model: Model to use for completion
        :param models: Models to use for completion
        :param frequency_penalty: Frequency penalty (-2.0 to 2.0)
        :param logit_bias: Token logit bias adjustments
        :param logprobs: Return log probabilities
        :param top_logprobs: Number of top log probabilities to return (0-20)
        :param max_completion_tokens: Maximum tokens in completion
        :param max_tokens: Maximum tokens (deprecated, use max_completion_tokens). Note: some providers enforce a minimum of 16.
        :param metadata: Key-value pairs for additional object information (max 16 pairs, 64 char keys, 512 char values)
        :param presence_penalty: Presence penalty (-2.0 to 2.0)
        :param reasoning: Configuration options for reasoning models
        :param response_format: Response format configuration
        :param seed: Random seed for deterministic outputs
        :param stop: Stop sequences (up to 4)
        :param stream: Enable streaming response
        :param stream_options: Streaming configuration options
        :param temperature: Sampling temperature (0-2)
        :param parallel_tool_calls:
        :param tool_choice: Tool choice configuration
        :param tools: Available tools for function calling
        :param top_p: Nucleus sampling parameter (0-1)
        :param debug: Debug options for inspecting request transformations (streaming only)
        :param image_config: Provider-specific image configuration options. Keys and values vary by model/provider. See https://openrouter.ai/docs/guides/overview/multimodal/image-generation for more details.
        :param modalities: Output modalities for the response. Supported values are \"text\", \"image\", and \"audio\".
        :param cache_control: Enable automatic prompt caching. When set, the system automatically applies cache breakpoints to the last cacheable block in the request. Currently supported for Anthropic Claude models.
        :param service_tier: The service tier to use for processing this request.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    @overload
    async def send_async(
        self,
        *,
        messages: Union[
            List[components.ChatMessages], List[components.ChatMessagesTypedDict]
        ],
        http_referer: Optional[str] = None,
        x_open_router_title: Optional[str] = None,
        x_open_router_categories: Optional[str] = None,
        provider: OptionalNullable[
            Union[
                components.ChatRequestProvider, components.ChatRequestProviderTypedDict
            ]
        ] = UNSET,
        plugins: Optional[
            Union[
                List[components.ChatRequestPluginUnion],
                List[components.ChatRequestPluginUnionTypedDict],
            ]
        ] = None,
        user: Optional[str] = None,
        session_id: Optional[str] = None,
        trace: Optional[
            Union[components.ChatRequestTrace, components.ChatRequestTraceTypedDict]
        ] = None,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[Dict[str, float]] = UNSET,
        logprobs: OptionalNullable[bool] = UNSET,
        top_logprobs: OptionalNullable[float] = UNSET,
        max_completion_tokens: OptionalNullable[float] = UNSET,
        max_tokens: OptionalNullable[float] = UNSET,
        metadata: Optional[Dict[str, str]] = None,
        presence_penalty: OptionalNullable[float] = UNSET,
        reasoning: Optional[
            Union[components.Reasoning, components.ReasoningTypedDict]
        ] = None,
        response_format: Optional[
            Union[components.ResponseFormat, components.ResponseFormatTypedDict]
        ] = None,
        seed: OptionalNullable[int] = UNSET,
        stop: OptionalNullable[
            Union[components.Stop, components.StopTypedDict]
        ] = UNSET,
        stream: Literal[True],
        stream_options: OptionalNullable[
            Union[components.ChatStreamOptions, components.ChatStreamOptionsTypedDict]
        ] = UNSET,
        temperature: OptionalNullable[float] = 1,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        tool_choice: Optional[
            Union[components.ChatToolChoice, components.ChatToolChoiceTypedDict]
        ] = None,
        tools: Optional[
            Union[
                List[components.ChatFunctionTool],
                List[components.ChatFunctionToolTypedDict],
            ]
        ] = None,
        top_p: OptionalNullable[float] = 1,
        debug: Optional[
            Union[components.ChatDebugOptions, components.ChatDebugOptionsTypedDict]
        ] = None,
        image_config: Optional[
            Union[
                Dict[str, components.ChatRequestImageConfig],
                Dict[str, components.ChatRequestImageConfigTypedDict],
            ]
        ] = None,
        modalities: Optional[List[components.Modality]] = None,
        cache_control: Optional[
            Union[components.CacheControl, components.CacheControlTypedDict]
        ] = None,
        service_tier: OptionalNullable[components.ChatRequestServiceTier] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStreamAsync[components.ChatStreamChunk]:
        r"""Create a chat completion

        Sends a request for a model response for the given chat conversation. Supports both streaming and non-streaming modes.

        :param messages: List of messages for the conversation
        :param http_referer: The app identifier should be your app's URL and is used as the primary identifier for rankings.
            This is used to track API usage per application.

        :param x_open_router_title: The app display name allows you to customize how your app appears in OpenRouter's dashboard.

        :param x_open_router_categories: Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param user: Unique user identifier
        :param session_id: A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 256 characters.
        :param trace: Metadata for observability and tracing. Known keys (trace_id, trace_name, span_name, generation_name, parent_span_id) have special handling. Additional keys are passed through as custom metadata to configured broadcast destinations.
        :param model: Model to use for completion
        :param models: Models to use for completion
        :param frequency_penalty: Frequency penalty (-2.0 to 2.0)
        :param logit_bias: Token logit bias adjustments
        :param logprobs: Return log probabilities
        :param top_logprobs: Number of top log probabilities to return (0-20)
        :param max_completion_tokens: Maximum tokens in completion
        :param max_tokens: Maximum tokens (deprecated, use max_completion_tokens). Note: some providers enforce a minimum of 16.
        :param metadata: Key-value pairs for additional object information (max 16 pairs, 64 char keys, 512 char values)
        :param presence_penalty: Presence penalty (-2.0 to 2.0)
        :param reasoning: Configuration options for reasoning models
        :param response_format: Response format configuration
        :param seed: Random seed for deterministic outputs
        :param stop: Stop sequences (up to 4)
        :param stream: Enable streaming response
        :param stream_options: Streaming configuration options
        :param temperature: Sampling temperature (0-2)
        :param parallel_tool_calls:
        :param tool_choice: Tool choice configuration
        :param tools: Available tools for function calling
        :param top_p: Nucleus sampling parameter (0-1)
        :param debug: Debug options for inspecting request transformations (streaming only)
        :param image_config: Provider-specific image configuration options. Keys and values vary by model/provider. See https://openrouter.ai/docs/guides/overview/multimodal/image-generation for more details.
        :param modalities: Output modalities for the response. Supported values are \"text\", \"image\", and \"audio\".
        :param cache_control: Enable automatic prompt caching. When set, the system automatically applies cache breakpoints to the last cacheable block in the request. Currently supported for Anthropic Claude models.
        :param service_tier: The service tier to use for processing this request.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    async def send_async(
        self,
        *,
        messages: Union[
            List[components.ChatMessages], List[components.ChatMessagesTypedDict]
        ],
        http_referer: Optional[str] = None,
        x_open_router_title: Optional[str] = None,
        x_open_router_categories: Optional[str] = None,
        provider: OptionalNullable[
            Union[
                components.ChatRequestProvider, components.ChatRequestProviderTypedDict
            ]
        ] = UNSET,
        plugins: Optional[
            Union[
                List[components.ChatRequestPluginUnion],
                List[components.ChatRequestPluginUnionTypedDict],
            ]
        ] = None,
        user: Optional[str] = None,
        session_id: Optional[str] = None,
        trace: Optional[
            Union[components.ChatRequestTrace, components.ChatRequestTraceTypedDict]
        ] = None,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[Dict[str, float]] = UNSET,
        logprobs: OptionalNullable[bool] = UNSET,
        top_logprobs: OptionalNullable[float] = UNSET,
        max_completion_tokens: OptionalNullable[float] = UNSET,
        max_tokens: OptionalNullable[float] = UNSET,
        metadata: Optional[Dict[str, str]] = None,
        presence_penalty: OptionalNullable[float] = UNSET,
        reasoning: Optional[
            Union[components.Reasoning, components.ReasoningTypedDict]
        ] = None,
        response_format: Optional[
            Union[components.ResponseFormat, components.ResponseFormatTypedDict]
        ] = None,
        seed: OptionalNullable[int] = UNSET,
        stop: OptionalNullable[
            Union[components.Stop, components.StopTypedDict]
        ] = UNSET,
        stream: Optional[bool] = False,
        stream_options: OptionalNullable[
            Union[components.ChatStreamOptions, components.ChatStreamOptionsTypedDict]
        ] = UNSET,
        temperature: OptionalNullable[float] = 1,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        tool_choice: Optional[
            Union[components.ChatToolChoice, components.ChatToolChoiceTypedDict]
        ] = None,
        tools: Optional[
            Union[
                List[components.ChatFunctionTool],
                List[components.ChatFunctionToolTypedDict],
            ]
        ] = None,
        top_p: OptionalNullable[float] = 1,
        debug: Optional[
            Union[components.ChatDebugOptions, components.ChatDebugOptionsTypedDict]
        ] = None,
        image_config: Optional[
            Union[
                Dict[str, components.ChatRequestImageConfig],
                Dict[str, components.ChatRequestImageConfigTypedDict],
            ]
        ] = None,
        modalities: Optional[List[components.Modality]] = None,
        cache_control: Optional[
            Union[components.CacheControl, components.CacheControlTypedDict]
        ] = None,
        service_tier: OptionalNullable[components.ChatRequestServiceTier] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.SendChatCompletionRequestResponse:
        r"""Create a chat completion

        Sends a request for a model response for the given chat conversation. Supports both streaming and non-streaming modes.

        :param messages: List of messages for the conversation
        :param http_referer: The app identifier should be your app's URL and is used as the primary identifier for rankings.
            This is used to track API usage per application.

        :param x_open_router_title: The app display name allows you to customize how your app appears in OpenRouter's dashboard.

        :param x_open_router_categories: Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param user: Unique user identifier
        :param session_id: A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 256 characters.
        :param trace: Metadata for observability and tracing. Known keys (trace_id, trace_name, span_name, generation_name, parent_span_id) have special handling. Additional keys are passed through as custom metadata to configured broadcast destinations.
        :param model: Model to use for completion
        :param models: Models to use for completion
        :param frequency_penalty: Frequency penalty (-2.0 to 2.0)
        :param logit_bias: Token logit bias adjustments
        :param logprobs: Return log probabilities
        :param top_logprobs: Number of top log probabilities to return (0-20)
        :param max_completion_tokens: Maximum tokens in completion
        :param max_tokens: Maximum tokens (deprecated, use max_completion_tokens). Note: some providers enforce a minimum of 16.
        :param metadata: Key-value pairs for additional object information (max 16 pairs, 64 char keys, 512 char values)
        :param presence_penalty: Presence penalty (-2.0 to 2.0)
        :param reasoning: Configuration options for reasoning models
        :param response_format: Response format configuration
        :param seed: Random seed for deterministic outputs
        :param stop: Stop sequences (up to 4)
        :param stream: Enable streaming response
        :param stream_options: Streaming configuration options
        :param temperature: Sampling temperature (0-2)
        :param parallel_tool_calls:
        :param tool_choice: Tool choice configuration
        :param tools: Available tools for function calling
        :param top_p: Nucleus sampling parameter (0-1)
        :param debug: Debug options for inspecting request transformations (streaming only)
        :param image_config: Provider-specific image configuration options. Keys and values vary by model/provider. See https://openrouter.ai/docs/guides/overview/multimodal/image-generation for more details.
        :param modalities: Output modalities for the response. Supported values are \"text\", \"image\", and \"audio\".
        :param cache_control: Enable automatic prompt caching. When set, the system automatically applies cache breakpoints to the last cacheable block in the request. Currently supported for Anthropic Claude models.
        :param service_tier: The service tier to use for processing this request.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.SendChatCompletionRequestRequest(
            http_referer=http_referer,
            x_open_router_title=x_open_router_title,
            x_open_router_categories=x_open_router_categories,
            chat_request=components.ChatRequest(
                provider=utils.get_pydantic_model(
                    provider, OptionalNullable[components.ChatRequestProvider]
                ),
                plugins=utils.get_pydantic_model(
                    plugins, Optional[List[components.ChatRequestPluginUnion]]
                ),
                user=user,
                session_id=session_id,
                trace=utils.get_pydantic_model(
                    trace, Optional[components.ChatRequestTrace]
                ),
                messages=utils.get_pydantic_model(
                    messages, List[components.ChatMessages]
                ),
                model=model,
                models=models,
                frequency_penalty=frequency_penalty,
                logit_bias=logit_bias,
                logprobs=logprobs,
                top_logprobs=top_logprobs,
                max_completion_tokens=max_completion_tokens,
                max_tokens=max_tokens,
                metadata=metadata,
                presence_penalty=presence_penalty,
                reasoning=utils.get_pydantic_model(
                    reasoning, Optional[components.Reasoning]
                ),
                response_format=utils.get_pydantic_model(
                    response_format, Optional[components.ResponseFormat]
                ),
                seed=seed,
                stop=stop,
                stream=stream,
                stream_options=utils.get_pydantic_model(
                    stream_options, OptionalNullable[components.ChatStreamOptions]
                ),
                temperature=temperature,
                parallel_tool_calls=parallel_tool_calls,
                tool_choice=utils.get_pydantic_model(
                    tool_choice, Optional[components.ChatToolChoice]
                ),
                tools=utils.get_pydantic_model(
                    tools, Optional[List[components.ChatFunctionTool]]
                ),
                top_p=top_p,
                debug=utils.get_pydantic_model(
                    debug, Optional[components.ChatDebugOptions]
                ),
                image_config=image_config,
                modalities=modalities,
                cache_control=utils.get_pydantic_model(
                    cache_control, Optional[components.CacheControl]
                ),
                service_tier=service_tier,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/chat/completions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream" if stream else "application/json",
            http_headers=http_headers,
            _globals=operations.SendChatCompletionRequestGlobals(
                http_referer=self.sdk_configuration.globals.http_referer,
                x_open_router_title=self.sdk_configuration.globals.x_open_router_title,
                x_open_router_categories=self.sdk_configuration.globals.x_open_router_categories,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.chat_request, False, False, "json", components.ChatRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="sendChatCompletionRequest",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "404",
                "408",
                "413",
                "422",
                "429",
                "4XX",
                "500",
                "502",
                "503",
                "524",
                "529",
                "5XX",
            ],
            stream=True,
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            return unmarshal_json_response(
                components.ChatResult, http_res, http_res_text
            )
        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStreamAsync(
                http_res,
                lambda raw: utils.unmarshal_json(
                    raw, operations.SendChatCompletionRequestResponseBody
                ).data,
                sentinel="[DONE]",
                client_ref=self,
            )
        if utils.match_response(http_res, "400", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res, http_res_text
            )
            raise errors.BadRequestResponseError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "401", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res, http_res_text
            )
            raise errors.UnauthorizedResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "402", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.PaymentRequiredResponseErrorData, http_res, http_res_text
            )
            raise errors.PaymentRequiredResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "404", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.NotFoundResponseErrorData, http_res, http_res_text
            )
            raise errors.NotFoundResponseError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "408", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.RequestTimeoutResponseErrorData, http_res, http_res_text
            )
            raise errors.RequestTimeoutResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "413", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.PayloadTooLargeResponseErrorData, http_res, http_res_text
            )
            raise errors.PayloadTooLargeResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "422", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityResponseErrorData, http_res, http_res_text
            )
            raise errors.UnprocessableEntityResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "429", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.TooManyRequestsResponseErrorData, http_res, http_res_text
            )
            raise errors.TooManyRequestsResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "500", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res, http_res_text
            )
            raise errors.InternalServerResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "502", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.BadGatewayResponseErrorData, http_res, http_res_text
            )
            raise errors.BadGatewayResponseError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "503", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableResponseErrorData, http_res, http_res_text
            )
            raise errors.ServiceUnavailableResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "524", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.EdgeNetworkTimeoutResponseErrorData, http_res, http_res_text
            )
            raise errors.EdgeNetworkTimeoutResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "529", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.ProviderOverloadedResponseErrorData, http_res, http_res_text
            )
            raise errors.ProviderOverloadedResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.OpenRouterDefaultError(
            "Unexpected response received", http_res, http_res_text
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ShellServerToolType = Literal["shell",]


class ShellServerToolTypedDict(TypedDict):
    r"""Shell tool configuration"""

    type: ShellServerToolType


class ShellServerTool(BaseModel):
    r"""Shell tool configuration"""

    type: ShellServerToolType



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


DatetimeServerToolType = Literal["openrouter:datetime",]


class DatetimeServerToolParametersTypedDict(TypedDict):
    timezone: NotRequired[str]
    r"""IANA timezone name (e.g. \"America/New_York\"). Defaults to UTC."""


class DatetimeServerToolParameters(BaseModel):
    timezone: Optional[str] = None
    r"""IANA timezone name (e.g. \"America/New_York\"). Defaults to UTC."""


class DatetimeServerToolTypedDict(TypedDict):
    r"""OpenRouter built-in server tool: returns the current date and time"""

    type: DatetimeServerToolType
    parameters: NotRequired[DatetimeServerToolParametersTypedDict]


class DatetimeServerTool(BaseModel):
    r"""OpenRouter built-in server tool: returns the current date and time"""

    type: DatetimeServerToolType

    parameters: Optional[DatetimeServerToolParameters] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatcontentitems import ChatContentItems, ChatContentItemsTypedDict
from openrouter.types import BaseModel
from typing import List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


ChatUserMessageRole = Literal["user",]


ChatUserMessageContentTypedDict = TypeAliasType(
    "ChatUserMessageContentTypedDict", Union[str, List[ChatContentItemsTypedDict]]
)
r"""User message content"""


ChatUserMessageContent = TypeAliasType(
    "ChatUserMessageContent", Union[str, List[ChatContentItems]]
)
r"""User message content"""


class ChatUserMessageTypedDict(TypedDict):
    r"""User message"""

    role: ChatUserMessageRole
    content: ChatUserMessageContentTypedDict
    r"""User message content"""
    name: NotRequired[str]
    r"""Optional name for the user"""


class ChatUserMessage(BaseModel):
    r"""User message"""

    role: ChatUserMessageRole

    content: ChatUserMessageContent
    r"""User message content"""

    name: Optional[str] = None
    r"""Optional name for the user"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatcontenttext import ChatContentText, ChatContentTextTypedDict
from openrouter.types import BaseModel
from typing import List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


ChatSystemMessageRole = Literal["system",]


ChatSystemMessageContentTypedDict = TypeAliasType(
    "ChatSystemMessageContentTypedDict", Union[str, List[ChatContentTextTypedDict]]
)
r"""System message content"""


ChatSystemMessageContent = TypeAliasType(
    "ChatSystemMessageContent", Union[str, List[ChatContentText]]
)
r"""System message content"""


class ChatSystemMessageTypedDict(TypedDict):
    r"""System message for setting behavior"""

    role: ChatSystemMessageRole
    content: ChatSystemMessageContentTypedDict
    r"""System message content"""
    name: NotRequired[str]
    r"""Optional name for the system message"""


class ChatSystemMessage(BaseModel):
    r"""System message for setting behavior"""

    role: ChatSystemMessageRole

    content: ChatSystemMessageContent
    r"""System message content"""

    name: Optional[str] = None
    r"""Optional name for the system message"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .compoundfilter import CompoundFilter, CompoundFilterTypedDict
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Any, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


TypeFileSearch = Literal["file_search",]


FiltersType = Union[
    Literal[
        "eq",
        "ne",
        "gt",
        "gte",
        "lt",
        "lte",
    ],
    UnrecognizedStr,
]


Value1TypedDict = TypeAliasType("Value1TypedDict", Union[str, float])


Value1 = TypeAliasType("Value1", Union[str, float])


Value2TypedDict = TypeAliasType(
    "Value2TypedDict", Union[str, float, bool, List[Value1TypedDict]]
)


Value2 = TypeAliasType("Value2", Union[str, float, bool, List[Value1]])


class FileSearchServerToolFiltersTypedDict(TypedDict):
    key: str
    type: FiltersType
    value: Value2TypedDict


class FileSearchServerToolFilters(BaseModel):
    key: str

    type: Annotated[FiltersType, PlainValidator(validate_open_enum(False))]

    value: Value2


FiltersTypedDict = TypeAliasType(
    "FiltersTypedDict",
    Union[CompoundFilterTypedDict, FileSearchServerToolFiltersTypedDict, Any],
)


Filters = TypeAliasType(
    "Filters", Union[CompoundFilter, FileSearchServerToolFilters, Any]
)


Ranker = Union[
    Literal[
        "auto",
        "default-2024-11-15",
    ],
    UnrecognizedStr,
]


class RankingOptionsTypedDict(TypedDict):
    ranker: NotRequired[Ranker]
    score_threshold: NotRequired[float]


class RankingOptions(BaseModel):
    ranker: Annotated[Optional[Ranker], PlainValidator(validate_open_enum(False))] = (
        None
    )

    score_threshold: Optional[float] = None


class FileSearchServerToolTypedDict(TypedDict):
    r"""File search tool configuration"""

    type: TypeFileSearch
    vector_store_ids: List[str]
    filters: NotRequired[Nullable[FiltersTypedDict]]
    max_num_results: NotRequired[int]
    ranking_options: NotRequired[RankingOptionsTypedDict]


class FileSearchServerTool(BaseModel):
    r"""File search tool configuration"""

    type: TypeFileSearch

    vector_store_ids: List[str]

    filters: OptionalNullable[Filters] = UNSET

    max_num_results: Optional[int] = None

    ranking_options: Optional[RankingOptions] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["filters", "max_num_results", "ranking_options"]
        nullable_fields = ["filters"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, Nullable, UnrecognizedStr
from openrouter.utils import validate_open_enum
import pydantic
from pydantic import ConfigDict
from pydantic.functional_validators import PlainValidator
from typing import Any, Dict, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


OutputServerToolItemStatus = Union[
    Literal[
        "completed",
        "in_progress",
        "incomplete",
    ],
    UnrecognizedStr,
]


class OutputServerToolItemTypedDict(TypedDict):
    r"""A generic OpenRouter server tool output item"""

    type: str
    r"""Server tool type (e.g. openrouter:datetime, openrouter:web_search)"""
    status: OutputServerToolItemStatus
    id: NotRequired[str]


class OutputServerToolItem(BaseModel):
    r"""A generic OpenRouter server tool output item"""

    model_config = ConfigDict(
        populate_by_name=True, arbitrary_types_allowed=True, extra="allow"
    )
    __pydantic_extra__: Dict[str, Nullable[Any]] = pydantic.Field(init=False)

    type: str
    r"""Server tool type (e.g. openrouter:datetime, openrouter:web_search)"""

    status: Annotated[
        OutputServerToolItemStatus, PlainValidator(validate_open_enum(False))
    ]

    id: Optional[str] = None

    @property
    def additional_properties(self):
        return self.__pydantic_extra__

    @additional_properties.setter
    def additional_properties(self, value):
        self.__pydantic_extra__ = value  # pyright: ignore[reportIncompatibleVariableOverride]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatmessagecontentitemtext import (
    ChatMessageContentItemText,
    ChatMessageContentItemTextTypedDict,
)
from openrouter.types import BaseModel
from typing import List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


SystemMessageRole = Literal["system",]


SystemMessageContentTypedDict = TypeAliasType(
    "SystemMessageContentTypedDict",
    Union[str, List[ChatMessageContentItemTextTypedDict]],
)
r"""System message content"""


SystemMessageContent = TypeAliasType(
    "SystemMessageContent", Union[str, List[ChatMessageContentItemText]]
)
r"""System message content"""


class SystemMessageTypedDict(TypedDict):
    r"""System message for setting behavior"""

    role: SystemMessageRole
    content: SystemMessageContentTypedDict
    r"""System message content"""
    name: NotRequired[str]
    r"""Optional name for the system message"""


class SystemMessage(BaseModel):
    r"""System message for setting behavior"""

    role: SystemMessageRole

    content: SystemMessageContent
    r"""System message content"""

    name: Optional[str] = None
    r"""Optional name for the system message"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatjsonschemaconfig import ChatJSONSchemaConfig, ChatJSONSchemaConfigTypedDict
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ChatFormatJSONSchemaConfigType = Literal["json_schema",]


class ChatFormatJSONSchemaConfigTypedDict(TypedDict):
    r"""JSON Schema response format for structured outputs"""

    type: ChatFormatJSONSchemaConfigType
    json_schema: ChatJSONSchemaConfigTypedDict
    r"""JSON Schema configuration object"""


class ChatFormatJSONSchemaConfig(BaseModel):
    r"""JSON Schema response format for structured outputs"""

    type: ChatFormatJSONSchemaConfigType

    json_schema: ChatJSONSchemaConfig
    r"""JSON Schema configuration object"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatchoice import ChatChoice, ChatChoiceTypedDict
from .chatusage import ChatUsage, ChatUsageTypedDict
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


ChatResultObject = Literal["chat.completion",]


class ChatResultTypedDict(TypedDict):
    r"""Chat completion response"""

    id: str
    r"""Unique completion identifier"""
    choices: List[ChatChoiceTypedDict]
    r"""List of completion choices"""
    created: float
    r"""Unix timestamp of creation"""
    model: str
    r"""Model used for completion"""
    object: ChatResultObject
    system_fingerprint: Nullable[str]
    r"""System fingerprint"""
    service_tier: NotRequired[Nullable[str]]
    r"""The service tier used by the upstream provider for this request"""
    usage: NotRequired[ChatUsageTypedDict]
    r"""Token usage statistics"""


class ChatResult(BaseModel):
    r"""Chat completion response"""

    id: str
    r"""Unique completion identifier"""

    choices: List[ChatChoice]
    r"""List of completion choices"""

    created: float
    r"""Unix timestamp of creation"""

    model: str
    r"""Model used for completion"""

    object: ChatResultObject

    system_fingerprint: Nullable[str]
    r"""System fingerprint"""

    service_tier: OptionalNullable[str] = UNSET
    r"""The service tier used by the upstream provider for this request"""

    usage: Optional[ChatUsage] = None
    r"""Token usage statistics"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["service_tier", "usage"]
        nullable_fields = ["system_fingerprint", "service_tier"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .formatjsonobjectconfig import (
    FormatJSONObjectConfig,
    FormatJSONObjectConfigTypedDict,
)
from .formatjsonschemaconfig import (
    FormatJSONSchemaConfig,
    FormatJSONSchemaConfigTypedDict,
)
from .formattextconfig import FormatTextConfig, FormatTextConfigTypedDict
from openrouter.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


FormatsTypedDict = TypeAliasType(
    "FormatsTypedDict",
    Union[
        FormatTextConfigTypedDict,
        FormatJSONObjectConfigTypedDict,
        FormatJSONSchemaConfigTypedDict,
    ],
)
r"""Text response format configuration"""


Formats = Annotated[
    Union[
        Annotated[FormatTextConfig, Tag("text")],
        Annotated[FormatJSONObjectConfig, Tag("json_object")],
        Annotated[FormatJSONSchemaConfig, Tag("json_schema")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]
r"""Text response format configuration"""
